/*   

     getopt has probs for passing on args we don't want to 
     look for.  Do parsing of required args by hand.


     Printcap flags meaningful to this front-end are
   
     nf		produce "new EUCS format" for J and C fields
     nc		this is a non-chargeable queue of an expensive printer
     ed  	"expensive device" - special permission required

*/

#include <stdio.h>
#include <pwd.h>
#include <malloc.h>
#include <local/banner.h>
#ifdef PLP
#include <lp.h>
#endif

struct arg_ent {
char *ent;
struct arg_ent *next;
};


#define	DEF_LP		"lp"
#define EOL		( (struct arg_ent *) 0 )
#define FORMS_LENGTH	40

struct arg_ent *head_args, *pt_args;

int		num_args = 0;    /* num of args to pass-on - always one
                                  coz we always add printer ourselves */

char		**oor_args;	 /* args list passed to exec */
char		*printer_flag;
char            Jflag[BUFFER_SIZE], Cflag[BUFFER_SIZE];
char            forms[FORMS_LENGTH];
char            userinfo[USER_INFO_LENGTH];
char            hostname[HOST_LENGTH];
char		delivery[DELIVERY_LENGTH];
char            files[FILES_LENGTH];
struct passwd	*pw_entry;


main(argc, argv, envp)
int argc;
char *argv[];
char *envp[];
{

int		i = 1;           /* miss out argv[0] !!! */
char		*printer;
char		*getenv();
char		*cp;
char    	bp[1024];
int     	status;
int		new_format;

#ifdef PLP
  Std_environ(argc, argv, envp);
#endif
  printer =  ( char * ) 0;
  forms[0] = '\0'; 
  files[0]='\0';
  while ( i < argc ) {
     if ( ( argv[i][0] == '-' ) && ( argv[i][1] == 'H' ) ) {
       if ( !do_HH(argv) )
          exit(1);
       else
          do_exec(argv);
     } else
     if ( ( argv[i][0] == '-' ) && ( argv[i][1] == 'P' ) ) {
        if ( argv[i][2] )
           printer=argv[i]+2;
        else 
           printer=argv[++i];
     } else 
     if ( ( argv[i][0] == '-' ) && ( argv[i][1] == 'G' ) ) {
        if ( argv[i][2] )
           strncpy(forms, argv[i]+2, FORMS_LENGTH - 1 );
        else 
           strncpy(forms, argv[++i], FORMS_LENGTH - 1 );
        forms[FORMS_LENGTH-1] = '\0';
     } else
     if ( ( argv[i][0] == '-' ) && ( argv[i][1] == 'J' ) ) {
        if ( ! argv[i][2] )    /* if arg not "hard-on" skip next one */
           i++;
     } else
     if ( ( argv[i][0] == '-' ) && ( argv[i][1] == 'C' ) ) {
        if ( ! argv[i][2] )    /* if arg not "hard-on" skip next one */
           i++;
     } else
     if ( ( argv[i][0] == '-' ) && ( argv[i][1] == 'h' || argv[i][1] == 'Z') ) {
        ;      /* do nowt - ie throw away */
     } else {
        if ( !pass_on(argv[i]) )   /* catch-all */
           fprintf(stderr,"lpr: Failed to malloc for arg");
        else {
           if ( ( argv[i][0] != '-' ) 
              && ( !( argv[i-1][0] == '-' && argv[i-1][1] == 'T' ) ) ) 
              save_filename(files,argv[i]);
        }         
     }
     i++;
   }  /* while */
   

   if ( !printer ){
         if ( (printer = getenv("PRINTER") ) != NULL ) {
	    ; /* nowt */
         }
         else 
             printer = DEF_LP;   
   }


   /* 
     Ok we know where we are going - is it a printer this guy
     can access.
   */

   if ( ( pw_entry = getpwuid(getuid() ) ) == NULL ) { /* for gecos field */
      fprintf(stderr,"lpr: unknown user!!\n");
      exit(1);
   }

#ifdef PLP
   /*  Tailor_names(); */
   Readconf ();
  if((status = Get_pc_entry(printer, Status_pc_vars, Status_pc_len)) < 0){
#else
  if ( ( status=pgetent(bp, printer) ) < 0 ) {
#endif
      fprintf(stderr,"lpr: Cannot open printer description file.\n");
      exit(1);
  }

  if ( status == 0 ) {
      fprintf(stderr,"lpr: %s: unknown printer\n",printer);
      exit(1);
  }

   if ( gethostname(hostname,HOST_LENGTH - 1) == -1)
       strcpy(hostname,"lpr: unknown host!!");
   hostname[HOST_LENGTH - 1] = NULL;

  /* 1 = yes, 0 = no */
  
#ifdef PLP
   if (EU == 1)
#else
   if ( pgetflag("eu") == 1 )   /* new format */
#endif
      new_format = 1 ;
   else
      new_format = 0 ;      

#ifdef CHARGE
#ifdef PLP
  if (NC == 1) {
#else
  if ( pgetflag("nc") == 1 ) {
#endif
#ifdef DEBUG
        printf("[%s]\t[%s]\n",pw_entry->pw_name,hostname);
#endif
      if ( validate_user(pw_entry->pw_name,hostname) == 0 ){
        fprintf(stderr,"lpr: You do not have access to the non-chargeable queue %s\n",printer);
        exit(1);
      }
  }

/* 
   If an "expensive device" and we have specified a forms queue beginning
   with a "c" then check if they can.
*/

#ifdef PLP
  if (ED == 1) {
#else
  if ( pgetflag("ed") == 1 ) {
#endif
     if ( ( ( forms[0] ) && ( forms[0] != 'c' ) ) || ( !forms[0] ) )
        if ( !access_expensive(pw_entry->pw_name) ) {
           fprintf(stderr,"lpr: You do not have access to restricted device %s.\n", printer);
           exit(1);
        }
  }

#endif

   if ( !files[0] ) 
      strcpy(files, "stdin");
   files[FILES_LENGTH-1] = '\0';  /* softly softly stroke the lion ... */


   if ( ( cp = getenv("NAME")) == NULL) 
      strncpy(userinfo, pw_entry->pw_gecos, USER_INFO_LENGTH - 1);
    else
      strncpy(userinfo, cp, USER_INFO_LENGTH - 1);
   userinfo[USER_INFO_LENGTH - 1] = NULL;

   if ( ( cp = getenv("DELIVERY")) == NULL)
      strcpy(delivery,"No delivery point specified.");
   else
      strncpy(delivery, cp, DELIVERY_LENGTH - 1);
   delivery[DELIVERY_LENGTH - 1] = NULL;

  if ( ( printer_flag=( char *) malloc( strlen(printer) + 3 ) ) == NULL ) {
    fprintf(stderr,"lpr: failed to malloc\n");  
    exit(1);
  }

  sprintf(printer_flag,"-P%s",printer);
  printer_flag[strlen(printer)+2] = '\0';

/*
   pt_args=head_args;
   while( pt_args != EOL ) {
      printf("[%s] \n", pt_args->ent);
      pt_args=pt_args->next;
   }
*/

/* set up arg list in oor_args */
   if ( !setup_args() ) {
      fprintf(stderr,"lpr: Failed to malloc for args list.\n");
      exit(1);
   }

   if ( new_format ) {
     if ( !do_new() ) 
        exit(1);
   } else
      do_old();

   oor_args[2]=Cflag;
   oor_args[3]=Jflag;

   do_exec(oor_args);
}


pass_on(passer)
char *passer;
{
struct arg_ent *this_un;

   if ( head_args == EOL )
      if ( ( head_args=( struct arg_ent *) malloc( sizeof(struct arg_ent) ) ) == NULL ) 
         return(0);
      else {
         head_args->ent=passer;
         head_args->next=EOL;
         pt_args=head_args;
         num_args++;
         return(1);
   } else {
      if ( ( this_un=( struct arg_ent *) malloc( sizeof(struct arg_ent) ) ) == NULL ) 
         return(0);
      else {
           this_un->ent=passer;
           this_un->next=EOL;
           pt_args->next=this_un;
           pt_args=pt_args->next;
           num_args++;
           return(1);
      }
   }
}



save_filename(list,comp)
char *list, *comp;
{
static int current_length=0;
int        this_length;

   this_length = strlen(comp);
   if ( (current_length + this_length ) < FILES_LENGTH) {
      strncat(list, comp, this_length);
      strcat(list," ");
      current_length += this_length+1;
   } else {
      strncat(list, comp, FILES_LENGTH-(current_length));
      current_length = FILES_LENGTH;
   }
}

/*

   Allocate mem for oor_args which will be an argv for the exec.
   Create it num_args + 5 coz we will set
   oor_args[0] to printer command
   oor_args[1] to to be printerflag ( ie -Pprinter_name )
   oor_args[2] to be Jflag
   oor_args[3] to be Cflag
   oor_args[num_args-1] to be NULL

*/   

setup_args()
{
int i;

   if ( ( oor_args = ( char **) calloc(num_args+5, sizeof( char * ) ) ) == NULL ) 
      return(0);

   pt_args=head_args;
   for ( i=0; i< num_args; i++ ) {
      oor_args[i+4] = pt_args->ent;
      pt_args=pt_args->next;
   }
   oor_args[0] = "lpr";
   oor_args[1] = printer_flag;
   oor_args[num_args+4] = ( char * ) 0 ;
   return(1);
}

char *protect(s)
char *s;
{

	if (index(s, ',') == NULL)
		return(s);
	else {
		char work_string[1024];
		char *rs;
		int i = 1;

		work_string[0] = '"';
		while (*s != '\0')
			if ((work_string[i++] = *s++) == '"')
				work_string[i++] = '"';
		work_string[i++] = '"';
		work_string[i] = '\0';
		rs = malloc(i + 1);
		(void)strcpy(rs, work_string);
		return(rs);
	}
}

do_new()
{
char eucs_code[7];

    if ( !getcode(pw_entry->pw_name, eucs_code ) ) {
       fprintf(stderr,"lpr: can't find eucs_code\n");
       return(0);
    }

/* NB: any ascii string which contains a comma must be placed within
 * double quotes. Within any such quoted string, a double quote character
 * must itself be doubled.
 */


    sprintf(Jflag,"-JE=%s,N=%s,SF=%s,D=%s,V=1.0",
	protect(eucs_code), protect(userinfo),
	protect(files), protect(delivery));
    if ( forms[0] ) {
       strcat(Jflag,",F=");
       strcat(Jflag,forms);
    }

    sprintf(Cflag,"-CZ");
    return(1);
}

do_old()
{
   sprintf(Cflag,"%s%-15s%s","-C",hostname,MAGIC_STRING);
   if (forms[0] != NULL)
     sprintf(Jflag,"%s%-30s%s:%s:%s:%s","-J",files,FILES_DELIM,userinfo,delivery,forms);
   else
     sprintf(Jflag,"%s%-30s%s:%s:%s","-J",files,FILES_DELIM,userinfo,delivery);
}

do_exec(oor_argv)
char **oor_argv;
{
#ifdef DEBUG
int  i=0;
   while ( oor_argv[i] != (char *) 0 )
       printf("[%s] ", oor_argv[i++]);
   printf("\n");
printf("%d(%d) %d(%d)\n", getuid(), geteuid(), getgid(), getegid());
#endif
   execv(PRINTER_CMD,oor_argv);
   fprintf(stderr,"lpr: front end - exec failed!!\n");
#ifdef DEBUG
perror("lprf");
#endif
   exit(1);                                            /* if exec failed */
}



do_HH(argv)
char **argv;
{
int i;

   if ( ( pw_entry = getpwuid(getuid() ) ) == NULL ) { /* for gecos field */
      fprintf(stderr,"lpr: unknown user!!\n");
      return(0);
   }

   if ( !can_HH(pw_entry->pw_name) ) {
      fprintf(stderr,"lpr: cannot use -H\n");
      return(0);
   }

   if (  strncmp(argv[1], "-H", 2 ) != 0 ) { 
      fprintf(stderr,"lpr: badly formed arg list with -H\n");
      return(0);
   }

   /* Shift the args down to get rid of -H */
   i=1;
   while( argv[i] != NULL ) {
      argv[i]=argv[i+1];
      i++;
   }
   argv[0] = "lpr";
   
   return(1);
}
void 
cleanup (int signal)
{
    ;
}
