#!/bin/sh
#
#===========================================================================
#=                                                                         =
#=  (C) Copyright 1995, Computer Technology Institute (CTI)                =
#=                                                                         =
#=  Permission to use, copy, modify and distribute this software and       =
#=  its documentation for non-commercial use and without fee is hereby     =
#=  granted, provided that the above copyright notice appear in all        =
#=  copies and that both that copyright notice and this permission         =
#=  notice and warranty disclaimer appear in supporting documentation,     =
#=  and that the name of Computer Technology Institute (CTI) or any of     =
#=  its entities not be used in advertising or publicity pertaining to     =
#=  distribution of the software and its documentation without specific    =
#=  prior permission.                                                      =
#=                                                                         =
#=  Computer Technology Institute (CTI) disclaims all warranties with      =
#=  regard to this software and makes no representations about the         =
#=  suitability of this software and its documentation for any purpose.    =
#=  It is provided "as is" without expressed or implied warranty.          =
#=                                                                         =
#===========================================================================
#=                                                                         =
#=                        Project CTI-Print                                =
#=                                                                         =
#= File:                                                                   =
#=   accounting.sh                                                         =
#=                                                                         =
#= Synopsis:                                                               =
#=   This script increments the current pages field in printer_perms file. =
#=                                                                         =
#= Author:                                                                 =
#=   Panos Dimakopoulos, Systems Programmer,                               =
#=   Computer Technology Institute,                                        =
#=   Division of Computing Facilities,                                     =
#=   P.O. Box 1122,                                                        =
#=   261 10  Patras,                                                       =
#=   Greece                                                                =
#=   (e-mail: dimakop@cti.gr)                                              =
#=   Tel: +30 61 992061                                                    =
#=   Fax: +30 61 993973                                                    =
#=                                                                         =
#===========================================================================
#


#============================================================================
#= Modification History:
#= 	Revision 1.1	94/11/24	16:51:02
#=		 Creation.
#= 	Revision 1.2	94/11/24	16:58:35
#=		Now it just changes the line with the ordinal number passed
#=		by the invoking filter. It should not check for word matching
#=		as the requesting machine or/and user might be described 
#=		from a regular expression.
#= 	Revision 1.3	95/02/10	13:59:42
#=		Full CTI address.
#= 	Revision 1.4	95/02/14	13:54:45
#=		 New Copyright notice.
#= 	Revision 1.5	95/02/14	19:19:13
#=		 Temporary file gets created in /tmp dir.
#


# Called with parameters set as follows:
#	$1 the printer perms file complete path
#	$2 the machine name
#	$3 the user name
#	$4 the printer name
#	$5 the num of pages to add to the seventh field of the file
#	$6 the line (starting from 1) of the perms file to be updated

if [ $# -ne 6 ]
then
	echo Usage: $0 permsFile machine user printer pages line 2>&1
	exit 1
fi

PERMSFILE=$1
TMPPERMSFILE=/tmp/filter.$$

trap '/usr/bin/rm -f $TMPPERMSFILE ; exit 2' 1 2 3 15

awk "BEGIN 						{OFS=\"\\t\";}
	 			{if ( NR == $6 ) { \$8 = \$8 + $5; } }
							{ print \$0; }
" $PERMSFILE > $TMPPERMSFILE
if [ $? -eq 0 ]
then
	cp $TMPPERMSFILE $PERMSFILE
	/usr/bin/rm -f $TMPPERMSFILE
	exit 0
else
	exit 1
fi
