/*
 ===========================================================================
 =                                                                         =
 =  (C) Copyright 1995, Computer Technology Institute (CTI)                =
 =                                                                         =
 =  Permission to use, copy, modify and distribute this software and       =
 =  its documentation for non-commercial use and without fee is hereby     =
 =  granted, provided that the above copyright notice appear in all        =
 =  copies and that both that copyright notice and this permission         =
 =  notice and warranty disclaimer appear in supporting documentation,     =
 =  and that the name of Computer Technology Institute (CTI) or any of     =
 =  its entities not be used in advertising or publicity pertaining to     =
 =  distribution of the software and its documentation without specific    =
 =  prior permission.                                                      =
 =                                                                         =
 =  Computer Technology Institute (CTI) disclaims all warranties with      =
 =  regard to this software and makes no representations about the         =
 =  suitability of this software and its documentation for any purpose.    =
 =  It is provided "as is" without expressed or implied warranty.          =
 =                                                                         =
 ===========================================================================
 =                                                                         =
 =                        Project CTI-Print                                =
 =                                                                         =
 = File:                                                                   =
 =   args.c                                                                =
 =                                                                         =
 = Synopsis:                                                               =
 =   This file contains subsidiary functions which process the args passed =
 =   to the filter and which update the accounting file too.               =
 =                                                                         =
 = Author:                                                                 =
 =   Panos Dimakopoulos, Systems Programmer,                                =
 =   Computer Technology Institute,                                        =
 =   Division of Computing Facilities,                                     =
 =   P.O. Box 1122,                                                        =
 =   261 10  Patras,                                                       =
 =   Greece                                                                =
 =   (e-mail: dimakop@cti.gr)                                              =
 =   Tel: +30 61 992061                                                    =
 =   Fax: +30 61 993973                                                    =
 =                                                                         =
 ===========================================================================
*/


/****************************************************************************
 * Modification History:
 * 	Revision 1.1	95/01/12	11:45:35
 *		 Creation.
 * 	Revision 1.2	95/01/25	15:29:10
 *		 Line to be updated in printer_perms file is passed with -Y
 *		 now and not with -L. -L is the flag for banner printing.
 *		 malloc() is used now to get space for the variables which
 *		 are enclodes in double quotes. This was necessary because,
 *		 if otherwise, all the strings passed enclosed in quotes
 *		 would be stored in the same place.
 * 	Revision 1.3	95/01/31	17:20:29
 *		 -B"<banner page title>" flag added.
 * 	Revision 1.4	95/02/10	16:42:47
 *		 Full CTI address.
 * 	Revision 1.5	95/02/14	14:04:10
 *		 New Copyright notice.
 * 	Revision 1.6	95/03/03	17:58:28
 *		 Embedding quotes removed for multiple word strings.
 * 	Revision 1.8	95/08/25	16:28:29
 *		 Porting to Solaris.
 *
 */

#include <stdio.h>
#include <sys/wait.h>
#include <malloc.h>
#include <hp4.h>

static char idargs[] = "@(#) args.c 95/08/25 - V 1.8" ;
extern char *name;		/* name of filter */
/* 
 * The following variables are set by the arguments passed to the filter
 * from the caller (lpd)
 */
	extern	char	*zopts, 
			*class, 
			*job, 
			*login, 
			*accntname, 
			*host, 
			*accntfile, 
			*bantitle, 
			*format,
			*printer, 
			*filename, 
			*username, 
			*date, 
			*seq_number, 
			*pr_commnt;
	extern	int	debug, 
			width, 
			length, 
			xwidth, 
			ylength, 
			literal, 
			indent, 
			prmsline;

extern int errorcode;

getargs(argc, argv)
	int argc;
	char **argv;
{
	int i;		/* argument index */
	char *arg;	/* argument */
	char flaglt;	/* flag letter */
	static char words[100];

	name = argv[0];

	for( i = 1; i < argc; ++i ){
		arg = argv[i];
		log(5,"filter arg=<%s>", arg);
		if( *arg == '-' ){ /* arg will be string */
				char *str;

				flaglt = arg[1]; /* Keep the option */
				strcat(words,&arg[2]);
				str = malloc(strlen(words)+1);
				strcpy( str, words );
				*(str + strlen(words)) = '\0' ;

				setvar( flaglt, str );
				words[0] = '\0' ;
		}
		else {
			/* must be accounting file */
			accntfile = arg;
		}
	}
}
			
#define INTV 0
#define STRV 1
#define FLGV 2
struct parm {
	int flag;
	char **var;
	int kind;
} Parmlist[] = {
{'B', &bantitle, STRV },
{'C', &class, STRV },
{'D', &date, STRV },
{'F', &format, STRV },
{'J', &job, STRV },
{'L', &username, STRV },
{'N', &filename, STRV },
{'P', &printer, STRV },
{'R', &accntname, STRV },
{'S', &pr_commnt, STRV },
{'X', &seq_number, STRV },
{'Y', (char **)&prmsline, INTV },
{'Z', &zopts, STRV },
{'c', (char **)&literal, FLGV },
{'d', (char **)&debug, INTV },
{'h', &host, STRV },
{'i', (char **)&indent, INTV },
{'l', (char **)&length, INTV },
{'n', &login, STRV },
{'w', (char **)&width, INTV },
{'x', (char **)&xwidth, INTV },
{'y', (char **)&ylength, INTV } };

int Parmlen = sizeof(Parmlist)/sizeof(struct parm);

/*
 * setvar( int flag, char *arg )
 * 1. look in table and find entry
 * 2. if STRV, then set 
 * 3. if INTV, then convert and set
 * 4. if FLGV, then set to 1
 */
setvar( flag, arg )
	int flag;
	char *arg;
{
	int u, l, i, c;	/* upper, lower, i */

	l = 0; u = Parmlen;
	while( l <= u ){
		i = (u+l)/2;
		c = flag - Parmlist[i].flag;
		if( 0 == c ){
			/* fprintf(stderr, "found parm %c, %d\n", flag, i ); */
			switch( Parmlist[i].kind ){
			case STRV: *Parmlist[i].var = arg; break;
			case INTV: *(int *)Parmlist[i].var = atoi(arg); break;
			case FLGV: *(int *)Parmlist[i].var = 1; break;
			}
			return;
		} else if( c < 0 ){
			/* fprintf(stderr, "down parm %c, %d\n", flag, i ); */
			u = i - 1 ;
		} else {
			/* fprintf(stderr, "up parm %c, %d\n", flag, i ); */
			l = i + 1 ;
		}
	}
	logerr(5,"Invalid option -%c", flag );
	return;
}

updaccnt(morepages)
int morepages;
{
	char HostNm[64], permsfile[512];
	int status, pid;

	if ( gethostname(HostNm, sizeof(HostNm)) < 0 ) {
		errorcode = FILTABORT;
		logerr_die(1,"I do not know my host name");
	}
	(void *)strcpy(permsfile,ETCPERMS);
	(void *)strcat(permsfile,HostNm);
	log(2,"Increments %s file with %d pages", permsfile, morepages);

	switch ( pid = fork() ) {
		char num1[4], num2[5];
		char accpath[150];
		case -1:
			errorcode = FILTABORT;
			logerr_die(1,"Cannot fork to update pages field");
		case  0:	/* child - Exec the script to update pages */
			sprintf(num1,"%3d",morepages);
			sprintf(num2,"%4d",prmsline);
			sprintf(accpath,"%s/%s",INSTDIR,ACCNTSH);
			execl(accpath,ACCNTSH,permsfile,host,login,printer,num1,num2,(char *)0);
			errorcode = FILTABORT;
			logerr_die(1,"Cannot exec %s to update pages field",ACCNTSH);
			exit(1);
		default:	/* father - Wait for the update
				 * Child exits with 0 for Ok, 2 if signal
				 * terminated and 1 otherwise.
				 */
			if ( wait(&status) ) {
				if (!(status & 0xffff)) /* Not signal, exit 0 */
					log(3,"File %s updated", permsfile);
				else {
					errorcode = FILTABORT;
					fatal(1,"Script %s failed",ACCNTSH);
				}
			}
		}
}
