/*
 ===========================================================================
 =                                                                         =
 =  (C) Copyright 1995, Computer Technology Institute (CTI)                =
 =                                                                         =
 =  Permission to use, copy, modify and distribute this software and       =
 =  its documentation for non-commercial use and without fee is hereby     =
 =  granted, provided that the above copyright notice appear in all        =
 =  copies and that both that copyright notice and this permission         =
 =  notice and warranty disclaimer appear in supporting documentation,     =
 =  and that the name of Computer Technology Institute (CTI) or any of     =
 =  its entities not be used in advertising or publicity pertaining to     =
 =  distribution of the software and its documentation without specific    =
 =  prior permission.                                                      =
 =                                                                         =
 =  Computer Technology Institute (CTI) disclaims all warranties with      =
 =  regard to this software and makes no representations about the         =
 =  suitability of this software and its documentation for any purpose.    =
 =  It is provided "as is" without expressed or implied warranty.          =
 =                                                                         =
 ===========================================================================
 =                                                                         =
 =                        Project CTI-Print                                =
 =                                                                         =
 = File:                                                                   =
 =   ifhp4.c                                                               =
 =                                                                         =
 = Synopsis:                                                               =
 =   This is the main driver module which handles the printer interface.   =
 =                                                                         =
 = Author:                                                                 =
 =   Panos Dimakopoulos, Systems Programmer,                                =
 =   Computer Technology Institute,                                        =
 =   Division of Computing Facilities,                                     =
 =   P.O. Box 1122,                                                        =
 =   261 10  Patras,                                                       =
 =   Greece                                                                =
 =   (e-mail: dimakop@cti.gr)                                              =
 =   Tel: +30 61 992061                                                    =
 =   Fax: +30 61 993973                                                    =
 =                                                                         =
 ===========================================================================
*/


/****************************************************************************
 * Modification History:
 * 	Revision 1.1	95/01/12	18:10:56
 *		 Creation.
 * 	Revision 1.2	95/01/13	16:13:25
 *		 sendjob() now has its personal signal (SIGIO) to be notified
 *		 when pipe is not empty.
 * 	Revision 1.3	95/01/16	15:52:44
 *		 Name of the trace file fixed to the correct /tmp/hp4.<pid>.
 * 	Revision 1.5	95/01/30	17:18:00
 *		 Landscape option added.
 * 	Revision 1.6	95/02/03	18:47:59
 *		 Nulls and sleep time decreased for speed.
 * 	Revision 1.7	95/02/06	20:07:13
 *		 Landscape and portrait strings revised together with
 *		 improving the margin and pitch settings.
 * 	Revision 1.8	95/02/08	18:50:16
 *		 After experimenting with an HP 4M the correct sleep time
 *		 found to be 5 secs for this model. So we keep this time
 *		 for both models to avoid complications with model definitions.
 * 	Revision 1.9	95/02/10	17:00:17
 *		 Full CTI address.
 * 	Revision 1.10	95/02/14	14:19:04
 *		 New Copyright notice.
 * 	Revision 1.11	95/05/30	17:46:06
 *		 Version 1.1 initiated.
 * 	Revision 1.13	95/08/23	14:22:12
 *		 Version 1.2 initiated with main feature the porting to Solaris.
 *		 Now if the required font is already downloaded it is not
 *		 downloaded again.
 *		 Font path can be specified in the -Z args. For example
 *		 lpr -P<printer> -Z'landscape font=/absolute/only/path' .
 * 	Revision 1.14	95/08/23	15:15:45
 *		 Some warning messages suppressed.
 * 	Revision 1.15	95/08/23	15:45:47
 *		 If resource saving is not enabled then font should be
 *		 always downloaded as the printer forgets it when it
 *		 changes personality.
 *
 */

#include <stdio.h>
#include <errno.h>
#include <signal.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/fcntl.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include <hp4.h>

static char idallf[] = "@(#) Computer Technology Institute, CTI-Print Project";
static char idhp4[] = "@(#)       Filter for HP LaserJet 4, V 1.2 \n" ;
static char idifhp4[] = "@(#) ifhp4.c 95/08/23 - V 1.15" ;

extern int errno;

char    sendline[MAXLINE+1];
char    readprin[MAXLINE+1];
char    job_start[100];
char    job_end[100];

#ifdef SVR4
#define CLEARBUF	memset(readprin, 0, MAXLINE+1)
#else
#define CLEARBUF	bzero(readprin, MAXLINE+1)
#endif
#define DIE		1

char *name;		/* name of filter */

int updatepfu = FALSE ;
char arr_fntname[150], *fntname = arr_fntname ;
char *pfu_fname = "pfu" ;	/* pfu is the file in the spool area that
				 * keeps the font path which has been
				 * successfully downloaded in the past.
				 */

/* 
 * The following variables are set by the arguments passed to the filter
 * from the caller (lpd)
 */
	char	*zopts, 
		*class, 
		*job, 
		*login, 
		*accntname, 
		*host, 
		*accntfile, 
		*format,
		*printer, 
		*filename, 
                *username,
                *date,
                *seq_number,
		*pr_commnt;
	int	width, 
		length, 
		xwidth, 
		ylength,
		debug, 
		literal, 
		indent;

int	prmsline,			/* line of printer_perms to change  */
	errorcode	= FILTABORT,	/* status the errlog funcs exit with */
	mode		= 0,		/* PCL or Postscript */
	npages		=-1, 		/* number of pages */
	pipefd[2],			/* Pipe between father and child which
					 * monitors the printer port */
	alrmflag,			/* Set by ALARM signal handling funcs */
	monitpid	=-1, 		/* pid of the port monitoring child */
	printer_ok,
	interrupted	= FALSE,	/* Set when signal received	*/
	frompipe	= FALSE,	/* Set when pipe is not empty.
					 * Used when the job itself is sent */
	job_started 	= FALSE,	/* Flag for the successful or not 
					 * initiation of the job */
	job_ended 	= FALSE;	/* Flag for the successful or not 
					 * termination of the job */
char	*query[3];			/* Holds the every time pjl commands 
					 * sent to the printer port */

static int	wait_time	= 30,	/* Time to wait between two retries 
					   (secs) */
		retries		= 0;	/* Number of retries if printer comes 
					   off-line 0 means for ever */

main(argc,argv)
int argc;
char *argv[];
{
	int	printer_code, k;
	char 	s[150];
	void 	segmviol(), cleanup(), nochildren();
	int 	readpipe();
	short 	wrap();

	debug = 2;
	get_retries();
	getargs( argc, argv );
	getpjlargs();

	(void *)signal(SIGCHLD, nochildren);
	(void *)signal(SIGINT, cleanup);
	(void *)signal(SIGHUP, cleanup);
	(void *)signal(SIGTERM, cleanup);
	(void *)signal(SIGQUIT, cleanup);
	(void *)signal(SIGBUS, cleanup);
	(void *)signal(SIGSEGV, cleanup);
	(void *)signal(SIGPIPE, cleanup);

	forkport();

   	/*
	* Try to synchronise with printer
	*/
	if ( pr_synch(STDOUT)==FALSE ) {
			log(2,"Failed to synchronise with printer");
			fexit(FILTRETRY);
	} else	log(2,"Synchronised with printer");

   	/*
	* Give status info only when you are asked to
	*/
	pr_ustatusoff(STDOUT);

       /*
	* Check printer status
	*/
	printer_ok = FALSE;
	do {
		switch ( printer_code=pr_status(STDOUT) )	{
			case 10001:		/* Printer ready */
				log(2,"-- Printer ready");
				printer_ok = TRUE;
				break;
			case 10003:		/* Printer warming up */
				log(2,"-- Printer warming up");
				sleep(45);	/* Warming up max time */
				break;
			case 10000:		/* Printer in powersave mode */
			case 35078:		
				log(2,"-- Printer in powersave");
				wake_up();
				/* sleep(45);	Warming up max time */
				break;
			case 10006:		/* Printer ready */
				log(2,"-- Toner low but carry on!");
				printer_ok = TRUE;
				break;
			default:
				check_code( printer_code, stderr );
				sleep(1);
				break;
		}
	} while	( printer_ok==FALSE );

	pr_ustatus(STDOUT,"@PJL USTATUS JOB = ON\r\n");
	
	pr_startjob(STDOUT);

	pr_ustatus(STDOUT,"@PJL USTATUS DEVICE = VERBOSE\r\n");
	pr_timed(STDOUT);
	sendjob(stdin, STDOUT );

	if ( mode == PCL )	pclresetprinter();
	if ( mode == POSTSCRIPT )	eofprinter();

	pr_endjob(STDOUT);
	check_job_end();

	check_pages();
	doaccnt();

	close(STDOUT);

	fexit(FILTSUCC);
}

int writecn(fd,ptr,nbytes)
register int fd;
register char *ptr;
int nbytes;
{
	int nleft, nwritten;
	void (*f)(), wait_printer();
	
	nleft = nbytes;
	while (nleft > 0) {
		f=signal(SIGALRM,wait_printer);
		alarm(3);
		nwritten = write (fd, ptr, nleft);
		alarm(0);
		(void *)signal(SIGALRM,f);
		if (nwritten == 0) {
			logerr(4,"writecn wrote nothing");
			continue ;
		}
		if (nwritten < 0) {
			logerr(4,"writecn failure");
			if (errno==EINTR) {
				log(4,"Due to signal");
				return(-1);
			} else
				fexit(FILTRETRY);
		}
		nleft -= nwritten;
		ptr += nwritten;
	}
	return(nbytes - nleft);
}

void cleanup(sig)
int sig;
{
	log(4,"Signal %u received-entering cleanup", sig);

	switch (sig) {
		case SIGINT:		/* Sent by lpd to terminate job */
			log(3,"Job Interrupted... Job removed");
			if ( mode==POSTSCRIPT ) {
				/*
				 * Send PostScript Interrupt/EOF chars
				 */
				writecn(STDOUT,"\003", 1);  /* ctrl-C */
				writecn(STDOUT,"\004", 1);  /* ctrl-D */
			}
			else if ( mode==PCL )
				pclresetprinter();  /* Send EscE char */
			/*
			 * If mode is not set to any of the two values
			 * PCL/POSTSCRIPT we do not care for resetting
			 * as no printing has taken place.
			 */
			fexit(FILTABORT);
		case SIGCHLD:
		case SIGTERM:
		case SIGHUP:
		case SIGQUIT:
			log(3,"Job Interrupted but job remains...");
			if ( mode==POSTSCRIPT ) {
				/*
				 * Send PostScript Interrupt/EOF chars
				 */
				writecn(STDOUT,"\003", 1);  /* ctrl-C */
				writecn(STDOUT,"\004", 1);  /* ctrl-D */
			}
			else if ( mode==PCL )
				pclresetprinter();  /* Send EscE char */
			/*
			 * If mode is not set to any of the two values
			 * PCL/POSTSCRIPT we do not care for resetting
			 * as no printing has taken place.
			 */
			fexit(FILTRETRY);
		case SIGSEGV:
		case SIGBUS:
			signal(SIGCHLD,SIG_DFL);
				/* Otherwise father receives SIGCHLD as well
				 * when he kills the child.
				 */
		case SIGPIPE:
			log(3,"Job Interrupted but job remains...");
			fexit(FILTRETRY);
		default:
			log(4,"Signal Ignored");
			return;
	}
}

static char null_str[]={"\000\000\000\000\000\000\000\000"};
#define NULL_LEN	8

wake_up()
{
	short cou;
	static int   woken=FALSE;

#define WAKEUP_STR	"\033(8U\033(s1p24v0s0b4148T\033*p100x900YWarming Up \
\033*p100x1100Ythe Printer"

	log(2,"Warming Up Process");
	if ( woken==TRUE ) {
		log(4,"But printer already awake...");
		return;
	}
	woken = TRUE ;

	write_check(STDOUT, PCLON, "Cannot set PCL Mode", DIE);
	for ( cou=0; cou<1000; cou++ )
		if ( writecn(STDOUT,null_str,NULL_LEN) != NULL_LEN )
			logerr_die(3,"Cannot send nulls");
	sleep(10);
	write_check(STDOUT, WAKEUP_STR, "Cannot send WAKEUP string", DIE);
	sleep(5);
	pclresetprinter();
	write_check(STDOUT, UELPJL, "Cannot send UELPJL", DIE);
}

/*
 * The transition is a bit odd. It needs to be accompanied with some
 * nulls and the driver hold off for some time. After experiments 
 * on HP 4M+ the optimum values have been found and used in the next
 * two functions.
 * The following comments from hp.com try to explain the situation:
>What is happening is the printer
>buffer fills with your switch command and the first parts of your job. The
>printer does not "prescan" the input looking for the <Esc>%-1234... but
>instead gets around to after some time has expired. During the switch,
>the data that you have already trasmitted is discarded and is picked up
>whereever your driver happens to be. For PostScript that's a messy
>situation.
>
>What you need to do is isolate the switch command. You can do this with
>nulls if you have no control over the timing. 8K of them will fill the
>largest allocated I/O buffer and then hold off the UNIX system. The
>switch will be made and the initial remaining nulls will be discarded. If
>you can control the timing of the data, you'll have to experiment with
>the correct time.
>
 * Although 8K is mentioned my experiments worked for just 240 nulls.
 * I cannot be very sure about this.
 */

setpcl_on()
{
	int cou;

	write_check(STDOUT, EOFSTR, "Cannot send EOF", DIE);
	resetprinter();
	for (cou=0;cou<1024;cou++)
		if ( writecn(STDOUT,null_str,NULL_LEN) != NULL_LEN )
			logerr_die(3,"Cannot send nulls");
	write_check(STDOUT, PCLON, "Cannot set PCL Mode", DIE);
	sleep(9);
}

setps_on()
{
	int cou;

	resetprinter();
	for (cou=0;cou<1024;cou++)
		if ( writecn(STDOUT,null_str,NULL_LEN) != NULL_LEN )
			logerr_die(3,"Cannot send nulls");
	write_check(STDOUT, POSTON, "Cannot set PostScript Mode", DIE);
	sleep(9);
}

eofprinter()
{
	write_check(STDOUT, EOFSTR, "Cannot send EOF char", 0);
}

resetprinter()
{
	write_check(STDOUT, UEL, "Cannot UEL reset printer", 0);
}

pclresetprinter()
{
	write_check(STDOUT, PCLRESETSTR, "Cannot PCL reset printer", 0);
}

deletefonts()
{
	write_check(STDOUT, DELFONTS, "Cannot delete fonts", 0);
}

/*
 * This function decides which font to download by reading 
 * MACHINESFONTS file.
 */
selectfont()
{
  FILE *pfu;    /* previous font used */
  char *s, *t, prevfnt[150], fulpathname[150];
  char *fnt_scan();
  int newfont();
  short resourcesave();

	if ( resourcesave() != ON ) update_pfu( pfu_fname, " " );
       /* find font for current job */
  if (zopts != NULL)
  {
        if ((s=strstr(zopts,"font")) != NULL)
        {
                log(1,"-Zfont option detected");
                t=strchr(s,'='); t++;
                log(1,"Step t=strchr()");
                if ((*t) == '/') {
                        sscanf(t,"%s",fntname);
			log(1,"font=[%s]",fntname);
                } else {
			sprintf(fntname,"%s/%s",FONTPATH,DEFLTFONT);
			logerr(1,"Only absolute paths accepted - Not: <%s>",t);
		}
        }
        else if ((s=fnt_scan(host)) == NULL)
           sprintf(fntname,"%s/%s",FONTPATH,DEFLTFONT);
        else if (strcmp(s,"NONE"))
           sprintf(fntname,"%s/%s",FONTPATH,s);
        else   /* when s="NONE" */
        {
           if (writecn(STDOUT, PGRESET, strlen(PGRESET)) != strlen(PGRESET))
             logerr(3,"Cannot send PGRESET string");
           return;
        };
  }
  else if ((s=fnt_scan(host)) == NULL)
        sprintf(fntname,"%s/%s",FONTPATH,DEFLTFONT);
  else if (strcmp(s,"NONE"))
        sprintf(fntname,"%s/%s",FONTPATH,s);
  else   /* when s="NONE" */
  {
        if (writecn(STDOUT, PGRESET, strlen(PGRESET)) != strlen(PGRESET))
            logerr(3,"Cannot send PGRESET string");
        return;
  };
        /* find font used on last job */
  sprintf(fulpathname,"%s",pfu_fname);
  fprintf(stderr,"fulpathname=[%s]\n",fulpathname); fflush(stderr);
  if ( (pfu=fopen(fulpathname,"r")) == NULL)
  {
        logerr(3,"Cannot open pfu file");
        newfont(fulpathname,fntname);
  }
  else if (fscanf(pfu,"%s\n",prevfnt) <= 0)
  {
        logerr(3,"Cannot read previously used font");
        newfont(fulpathname,fntname);
  }
  else    /* open and read successful */
  {
        fclose(pfu);

                /* if fonts different, send new font, else do nothing */
        if (strcmp(prevfnt,fntname) !=0 )
        {
           fprintf(stderr,"different fonts detected\n"); fflush(stderr);
           newfont(fulpathname,fntname);
        }
        else
        {
           log(3 ,"same fonts detected\n");
           if (writecn(STDOUT, SELFONT, strlen(SELFONT)) != strlen(SELFONT))
           {
                log(2,"Cannot send SELFONT string");
                newfont(fulpathname,fntname);
           }
           else
           {
                log(3,"Font selected successfully");
                updatepfu=FALSE;
           };
        }
  }
};


set_mode(m)
short m;
{
	if( m == POSTSCRIPT ) {
		mode = POSTSCRIPT ;
		log(3,"Entering POSTSCRIPT Mode");
		setps_on();
	} else {
		mode = PCL ;
		log(3,"Entering PCL Mode");
		setpcl_on();
		page_set();
	}
}

page_set()
{
	char s[150], lnd='0', *ptr, *spitch;


	write_check(STDOUT, CRLFSTR, "Cannot send CRLF string", 0);

	selectfont();

	if ( zopts ) {
		char *pptr = PGSTRDPL;
		if ( strstr(zopts, "landscape")!=NULL ) {
			lnd = '1' ;
			width = 132 ;
			length = 41 ;
			pptr = PGSTR;
			sprintf(s, PGSTR, lnd, width, length);
		}
		if ( strstr(zopts, "duplex")!=NULL ) {
			sprintf(s, pptr, lnd, width+5, length);
		}
	} else
		sprintf(s, PGSTR, lnd, width, length);

	if ( writecn(STDOUT,s,strlen(s)) != strlen(s) ) {
		logerr(3,"Cannot send Control Page string");
	}

	if ( width != 132 )	width = 80;

	if ( zopts && (strstr(zopts,"landscape")!=NULL) )	ptr = PITCHLND;
	else {
		if ( width==132 )	ptr = PITCH132;
		else			ptr = PITCH80;
	}

	if ( writecn(STDOUT,ptr,strlen(ptr)) != strlen(ptr) )
		logerr(3,"Cannot send PITCH80 string");

	if ( zopts && (strstr(zopts,"duplex")!=NULL) ) 		ptr = SHIFTDPL;
	else 							ptr = SHIFT;

	if ( writecn(STDOUT,ptr,strlen(ptr)) != strlen(ptr) )
		logerr(3,"Cannot send SHIFT string");

	wrap_set();
}

/*
 * If line wrapping is required the WRAP env variable should be set to ON.
 */
wrap_set()
{
	short wrap();

	if ( wrap() == ON )
		write_check(STDOUT, WRAPON, "Cannot send WRAPON string", 0);
}

/*
 * Unfortunately the series 4 printers cannot always keep the downloaded
 * fonts permanently. They can only when they have the Resource Saving
 * option installed. Details about this can be obtained from the HP
 * LaserJet 4 Plus and 4M Plus Printer User's Manual" on page 5-9.
 * Therefore the RESRCSAVE env var in ifhp4.sh denotes if such an option
 * is in place.
 */
short resourcesave()
{
	char *s;

	s = getenv("RESRCSAVE") ;
	if ( ! s )			return ( OFF );
	if ( ! strcmp(s,"ON") )		return ( ON );
		else			return ( OFF );
}

short wrap()
{
	char *s;

	s = getenv("WRAP") ;
	if ( ! s )			return ( OFF );
	if ( ! strcmp(s,"ON") )		return ( ON );
		else			return ( OFF );
}

open_trace()
{
	char *s;
	int fd;

	s = getenv("TRACE");
	if ( s ) {
		if (!strcmp(s,"ON")) {
			char ss[50];

			sprintf(ss,"/tmp/hp4.%04d", getpid());
			fd = open(ss, O_WRONLY|O_CREAT,0600);
			if (fd < 0) {
				log(2,"Cannot open trace file");
				return( 0 );
			} else	return( fd );
		} else	return ( 0 );
	} else
		return (0);
}

sendjob(fp, sockfd)
register FILE    *fp;
register int    sockfd;
{
	fd_set	writefds;
	short 	rdcnt, wrcnt;
	char 	*tmp;
	int 	tracefd;

	tracefd = open_trace();

	do {
		if (rdcnt=file_readline(fileno(fp), sendline, MAXLINE)) {
			if (tracefd)	write(tracefd, sendline, rdcnt);
			tmp = sendline;
			if (!mode) {
			    if (!strncmp("%!",sendline,2) || 
									!strncmp("\004%!",sendline,2)) {
						if ( sendline[0] == '\004' ) {tmp++ ; rdcnt-- ; }
						set_mode(POSTSCRIPT);
					} else set_mode(PCL);
					pr_duplex();
                }	/* if (!mode) */
                wrcnt=writecn(sockfd, tmp, rdcnt);
		if (wrcnt != rdcnt) {
			logerr_die(2,"Cannot write what read (%d bytes)",rdcnt);
			readpipe((int *)0);
        }
   		}  /* if file_readline */
    } while ( rdcnt != 0 );
	close(tracefd);
}

/*
 * The following function has been copied from a printer driver 
 * written by Lawrence LeMay (lemay@epx.cis.umn.edu).
 */
/*
 * Read a line from a descriptor.  Read the line one byte at a time,
 * looking for '\015' ('\r') (FF is the terminator for all responses returned 
 * from the series 4 printers).  We store the FF in the buffer,
 * then follow it with a null (the same as fgets(3)).
 * We return the number of characters up to, but not including,
 * the null (the same as strlen(3)).
 */

int
port_readline(fd, ptr, maxlen)
register int    fd;
register char   *ptr;
register int    maxlen;
{
    int                      n, rc;
    char                     c;
    static struct timeval    timeout;
    fd_set                   readfds;

    timeout.tv_sec = 1;
    timeout.tv_usec = 0;

    for (n = 0; n < maxlen; n++) {
        FD_ZERO(&readfds);
        FD_SET(fd, &readfds);
        if (select(fd+1, &readfds, (fd_set *) 0, (fd_set *) 0,
                &timeout) < 0) {
		if (errno==EINTR)	{
			log(4,"Select received signal");
			n-- ;
			continue;
		}
        	logerr(1,"select error");
        	fexit(FILTRETRY);
        }

        if (!FD_ISSET(fd, &readfds)) break;

        if ( (rc = read(fd, &c, 1)) == 1) {
		*ptr++ = c;
		if (c == '\014') {	/* ^L returned from printer */
			n++ ;
			break;
		}
        } else if (rc == 0) {
            if (n == 0)
                return(0);    /* EOF, no data read */
            else
                break;        /* EOF, some data was read */
        } else {
            return(-1);    /* error */
	}
    }

    *ptr = 0;
    return(n);
}

/*
 * Read a line from a descriptor.  Read the line one byte at a time,
 * looking for a newline.  We store the newline in the buffer,
 * then follow it with a null (the same as fgets(3)).
 * We return the number of characters up to, but not including,
 * the null (the same as strlen(3)).
 */

int
file_readline(fd, ptr, maxlen)
register int    fd;
register char   *ptr;
register int    maxlen;
{
    int                      n, rc;
    char                     c, prev='\0';

    for (n = 0; n < maxlen; n++) {

        if ( (rc = read(fd, &c, 1)) == 1) {
		*ptr++ = c;
		if (c == '\n') {
			n++ ;
			break;
		}
/*
		if ( (c == '\004') && (prev == '\003') ) {
			log(1,"Abort chars received...");
			cleanup(SIGINT);
		}
		prev = c;
*/
        } else if (rc == 0) {
            if (n == 0)
                return(0);    /* EOF, no data read */
            else
                break;        /* EOF, some data was read */
        } else {
            return(-1);    /* error */
	}
    }

    *ptr = 0;
    return(n);
}

/*
 *  doaccnt()
 *  writes the accounting information to the accounting file
 *  This has the format: user host printer pages format date
 */
doaccnt()
{
	time_t t, time();
	char *ctime();
	FILE *f;

	t = time((time_t *)0);
	if(accntfile && access(accntfile, W_OK) >= 0 &&
	    (f = fopen(accntfile, "a" )) != NULL) {
            fprintf(f,"%s\t%s\t%s\t%ld %s %s\t%s %s",
                    login? login: "NULL",
                    host? host: "NULL",
                    printer? printer: "NULL",
                    npages,
                    format? format: " ",
                    filename? filename: "NULL",
                    seq_number ? seq_number : "XXX",
                    ctime(&t));
		(void)fclose(f);
	}
}

readpipe(inptr)
int *inptr;
{
	char *buf, *getcrstr();
	int   n, cd, get_num();
	void wait_printer();

	CLEARBUF;
	if ( (n=port_readline(pipefd[0],readprin,MAXLINE)) <0 ) {
		CLEARBUF;
		logerr(2,"Could not read from pipe");
		return( -1 );
	} else {
		if ( n == 0 )	return n;
		log(3,"I have read from pipe <%s>", readprin);
	}

	if UNSOLCODE( getcrstr(readprin,2) ) {
		int i;

		cd		= get_num( getcrstr(readprin,2) );
		if ( inptr )
			 *inptr		= cd;
		CLEARBUF;
		if ( (i=check_code( cd, stderr )) == FILTRETRY ) {
			log(2,"Unsolicited problem with code =%d", cd);
			wait_printer(i);
		}
		return n;
	}
	if JOBSTART( getcrstr(readprin,3) ) {
		CLEARBUF;
		job_started	= TRUE;
		return n;
	}
	if JOBEND( getcrstr(readprin,3) ) {
		job_ended	= TRUE;
		npages		= get_num( getcrstr(readprin,4) );
		CLEARBUF;
		return n;
	}
	return n;

}

/*
 * The passed argument is supposed to be a string starting off with any
 * pattern (such as "PAGES=" or "CODE=") and followed by a number.
 * The number is returned as this is the one of major concern.
 */
get_num(s)
char *s;
{
	int i;

	if (!s) return(0);

	while ( *s && (!isdigit(*s)) )	s++ ;
	if ( *s )	sscanf(s,"%d",&i);	
		else  i=0;

	return( i );
}

/*
 * The child attends the printer port and when data turn up it
 * grabs it. Data is passed through a pipe to the father.
 */
forkport()
{

	void childdied(), slay();
	int readpipe();

	if ( pipe(pipefd) < 0 ) {
		logerr(1,"Cannot set pipe");
		fexit(FILTABORT);
	}

	monitpid = fork();
	log(1,"Right after fork with monitpid=%d", monitpid);
	switch (monitpid) {
		case -1:
			logerr(1,"Cannot fork process");
			fexit(FILTABORT);
			break;
		case 0:				/* child */
			log(4,"Child with pid=%d",getpid());
			close(pipefd[0]);	/* Child writes on p[1] */
			log(4,"Trying to set the signals");
			if ( signal(SIGTERM, slay) < 0) {
				logerr(2,"Signal cannot be set");
				fexit(FILTSUCC);
			}
			if ( signal(SIGPIPE, slay) < 0) {
				logerr(2,"Signal cannot be set");
				fexit(FILTSUCC);
			}
			if ( signal(SIGBUS,slay) < 0) {
				logerr(2,"Signal cannot be set");
				fexit(FILTSUCC);
			}
			if ( signal(SIGSEGV,slay) < 0) {
				logerr(2,"Signal cannot be set");
				fexit(FILTSUCC);
			}
			if ( signal(SIGQUIT, slay) < 0) {
				logerr(2,"Signal cannot be set");
				fexit(FILTSUCC);
			}
			if ( signal(SIGINT, slay) < 0) {
				logerr(2,"Signal cannot be set");
				fexit(FILTSUCC);
			}
			if ( signal(SIGHUP, slay) < 0) {
				logerr(2,"Signal cannot be set");
				fexit(FILTSUCC);
			}
			log(4,"Signals have been set ok");
			monitorport();		/* Infinite loop for reading */
			break;
		default:			/* father */
			log(2,"Father with pid=%d",getpid());
			if ( close(pipefd[1])<0 )
				logerr("Cannot close pipe");	
				/* Father reads from p[0] */
			else	log(1,"After closing pipe in father");
			return;
	}
}

/*
 * This is the process to monitor for ever the port attached to the printer.
 * When it reads something it sends it through the pipe to its father.
 */
monitorport()
{
	char buf[MAXLINE+1];
	int rdcnt;


	log(3,"Monitoring the printer port");

	for( ; ; ) {
#ifdef SVR4
		memset(buf, 0, MAXLINE);
#else
		bzero(buf,MAXLINE);
#endif
		rdcnt=port_readline(STDOUT, buf, MAXLINE);

		if (rdcnt == -1)	{
			logerr(3,"Cannot read from port");
			continue;
		}

		if (rdcnt == 0)	continue;

		log(3,"I have read <%s> from printer",buf);

		if (writecn(pipefd[1],buf,strlen(buf))<0) {
			logerr(3,"...but cannot write it on pipe");
			fexit(FILTRETRY);
		}
	}/* for loop */
}

void slay(sig)
int sig;
{
	log(4,"Signal %u received", sig);
	fexit(FILTSUCC);
}

void nochildren()
{
	log(4,"SIGCHLD received by father");
	if ( job_ended == TRUE )
		fexit(FILTSUCC);
	else {
		log(3,"Calling cleanup...");
		cleanup(SIGCHLD);
		fexit(FILTRETRY);
	}
}

void pr_late()
{
	log(5,"In pr_late()");
	alrmflag = 1;
}

pr_query(fd, n)
int  fd, n;
{
	short i, len ;

	for (i=0; i<n; i++) {
		len = strlen(query[i]);
		if (  writecn(fd, query[i], len) != len  )	{
			logerr(1,"Cannot write query string <%s>", query[i]);
			fexit(FILTABORT);
		} else
			log(3,"Writing query string <%s>", query[i]);
	}
}

/*
 * In the PJL manual, Chapter 9, Section: Determining if printer status
 * readback is available it states that if the application does not
 * receive a response within three seconds, then the application can 
 * assume printer status readback is currently unavailable.
 */
pr_synch(fd)
int fd;
{
	int synccont;
	int   synchronised = FALSE, blmask, fftries = 0;
#ifdef SVR4
        char buf[MAXLINE+1];
#else
        char *sprintf(), buf[MAXLINE+1];
#endif
	void (*signal())(), (*f)(), pr_late();

	log(2,"Synchronisation attempt");

	query[0]="\033%-12345X@PJL\r\n";
#ifdef SVR4
        sprintf (buf,"@PJL ECHO %d\r\n",getpid());
        query[1] = buf;
#else
        query[1]=sprintf(buf,"@PJL ECHO %d\r\n",getpid());
#endif
	pr_query(fd, 2);

	f=signal(SIGALRM,pr_late);
	alrmflag=0;
	alarm(60);

	do {
		log(4, "fftries=%d", ++fftries);
		if ( readpipe((char *)0) <= 0 )	continue;

		if ( !strncmp(readprin,buf,strlen(buf)-2) ) {
			synchronised = TRUE;
			alarm(0);
			break;
		}

		if ( strstr(readprin, "@PJL ECHO") ) {
			log(2,"Synchro input from previous attempt <%s>", readprin);
		}

	} while ( !alrmflag );

	signal(SIGALRM,f);

	return ( synchronised );
}

pr_status(fd)
int fd;
{
	int st=0, fftries = 0;
	char *str;

	log(2,"Asking for printer status");

	query[0]="@PJL INFO STATUS\r\n";
	pr_query(fd, 1);

	signal(SIGALRM,pr_late);
	alrmflag=0;
	alarm(120);

	do {
		log(4, "fftries=%d", ++fftries);
		readpipe(&st);
		if ( st > 0 )	{
			alarm(0);
			return( st );
		}

	} while ( !alrmflag );

	signal(SIGALRM,SIG_DFL);
	return( st );
}

pr_timed(fd)
int fd;
{
#ifdef SVR4
        char buf[MAXLINE+1];
#else
        char *sprintf(), buf[MAXLINE+1];
#endif
 
#ifdef SVR4
        sprintf(buf,"@PJL USTATUS TIMED = %d \r\n", wait_time);
        query[0] = buf;
#else
        query[0]=sprintf(buf,"@PJL USTATUS TIMED = %d \r\n", wait_time);
#endif
        pr_query(fd, 1);
}

pr_ustatusoff(fd)
int fd;
{
	query[0]="@PJL USTATUSOFF\r\n";
	pr_query(fd, 1);
}

pr_ustatus(fd,pjlstr)
int fd;
char *pjlstr;
{
	query[0]=pjlstr;
	pr_query(fd, 1);
}

pr_startjob(fd)
int fd;
{
#ifdef SVR4
        char buf[MAXLINE+1];
#else
        char *sprintf(), buf[MAXLINE+1];
#endif
#ifdef SVR4
        sprintf(buf,"@PJL JOB %s", job_start);
        query[0] = buf;
#else
        query[0]=sprintf(buf,"@PJL JOB %s", job_start);
#endif
        pr_query(fd, 1);
}

pr_endjob(fd)
int fd;
{
#ifdef SVR4
        char buf[MAXLINE+1];
#else
        char *sprintf(), buf[MAXLINE+1];
#endif
 
        query[0]= UELPJL ;
#ifdef SVR4
        sprintf(buf,"@PJL EOJ %s", job_end);
        query[1] = buf;
#else
        query[1]=sprintf(buf,"@PJL EOJ %s", job_end);
#endif
        query[2]= UEL ;
 
        pr_query(fd, 3);
}

check_job_end()
{
	int fftries = 0;

	log(2,"Checking for job end");

	signal(SIGALRM,pr_late);
	alrmflag=0;
	alarm(15*60);

	while ( job_ended!=TRUE ) {
		log(4, "fftries=%d", ++fftries);
		readpipe((char *)0);

		if (alrmflag)	break;

		if ( job_ended == TRUE ) {	
			alarm(0);
			log(2,"Job ended successfully");
			return;
		} 
	}

	log(1,"Job ended unsuccessfully");
	fexit(FILTRETRY);
}

check_pages()
{
	int fftries = 0;

	log(2,"Checking for job end");

	signal(SIGALRM,pr_late);
	alrmflag=0;
	alarm(60);

	while (npages < 0) {
		log(4, "fftries=%d", ++fftries);
		readpipe((char *)0);

		if ( job_ended == TRUE ) {	
			if (alrmflag) {
				log(1,"Could not get page count");
				fexit(FILTABORT);
			}
		} 
	}

	alarm(0);
	log(2,"Page count returned=%d", npages);
	updaccnt(npages);
	return;

}

getpjlargs()
{
	sprintf(job_start,"NAME=\"Start of CTI-Print Job: %d\"\r\n",getpid());
	sprintf(job_end,"NAME=\"End of CTI-Print Job: %d\"\r\n", getpid() );
}

fexit(i)
int i;
{
	log(1,"Exit value = %d", i);
	if ( monitpid > 0 )	{
		int chld;

		log(3,"Father murders %d", monitpid);
		if ( kill(monitpid,SIGTERM) < 0 )
			logerr(3,"...but killing unsuccessful");
		chld = wait((int *)0);
		log(3,"wait returned %d", chld);
		if (updatepfu)	update_pfu( pfu_fname, fntname );
	}
	exit(i);
}

check_code( cd, fp )
int cd;
FILE *fp;
{
	log(5,"In check_code (status %d)",cd);

	if ( cd == 10001 ) {
		log(2,"Printer should be OK (status %d)",cd);
		return(FILTSUCC);
	}

	if ( (cd == 10000) || (cd == 35078) ) {
		log(2,"Printer in Powersave (status %d)",cd);
		wake_up();
		return(FILTRETRY);
	}

	if ( cd == 10002 ) {
		log(2,"Printer should be offline (status %d)",cd);
		return(FILTRETRY);
	}

	if ( cd == 10003 ) {
		log(2,"Printer warming up (status %d)",cd);
		return(FILTRETRY);
	}

	if ( cd == 10006 ) {
		log(2,"Toner Low (status %d)",cd);
		return(FILTSUCC);
	}

	if ( cd <= 10099 ) {
		log(2,"$$$ $$$ $$$ (status %d)",cd);
		return(FILTRETRY);
	}

	if ( cd <= 11499 ) {
		log(2,"Background paper loading (status %d)",cd);
		return(FILTSUCC);
	}

	if ( (cd > 20000) && (cd <= 20030) ) {
		log(2,"PJL syntax error (status %d)",cd);
		return(FILTSUCC);
	}

	if ( cd == 25002 )  {
		log(2,"PJL prefix missing (status %d)",cd);
		return(FILTSUCC);
	}

	if ( ( cd > 25000 ) && ( cd < 25999 ) )  {
		log(2,"PJL syntax warning (status %d)",cd);
		return(FILTSUCC);
	}

	if ( (cd >= 20001) && (cd <= 35999) ) {
		log(2,"$$$ $$$ $$$ (status %d)",cd);
		fexit(FILTABORT);
	}

	if ( cd == 40021 ) {
		log(2,"Printer open (status %d)",cd);
		return(FILTRETRY);
	}

	if ( cd == 40022 ) {
		log(2,"Paper jam (status %d)",cd);
		return(FILTRETRY);
	}

	if ( cd == 40079 ) {
		log(2,"Printer manually taken offline (status %d)",cd);
		return(FILTRETRY);
	}

	if ( (cd >= 41001) && (cd <= 41999) ) {
		log(2,"Foreground paper loading (status %d)",cd);
		return(FILTRETRY);	/* Needs paper */
	}

	log(2,"Unknown reason");
	return(FILTRETRY);
}

void	segmviol(sig)
int sig;
{
	log(4,"Segmentation violation (Signal %d received)", sig);
	fexit(FILTRETRY);
}

void wait_printer(i)
int i;
{
	if ( retries == -10 )	{
		log(2,"Let's sleep for %d secs", wait_time);
		sleep( wait_time );
		return;
	}

	if ( retries-- > 0 )	{
		log(2,"Let's sleep for %d secs", wait_time);
		sleep( wait_time );
	} else {
		log(2,"Can not sleep anymore");
		cleanup(SIGTERM);
		fexit(i);
	}
}

write_check(fd,str,msg,flg)
int  	 fd,   flg;
char	*str, *msg;
{
	int len, logerr(), logerr_die(), (*f)();

	len = strlen(str);
	if ( flg==DIE )
		f = logerr_die;
	else
		f = logerr;

	if ( writecn(fd, str, len) != len )		(*f)(3,msg);
}

get_retries()
{
	char *s;
	int r;

	s = getenv("RETRIES") ;
	if ( s ) {
		sscanf(s,"%d", &r);
		retries = r;
		if ( retries == 0 )	retries = -10 ;
	}

	s = getenv("SLEEPTIME") ;
	if ( s ) {
		sscanf(s,"%d", &r);
		wait_time = r;
	}
}

#define PSDUPLEX "statusdict begin true setduplexmode end\r\n"
#define PCLDUPLEX "\033&l1S"

pr_duplex()
{
	char *s;

	if ( ! zopts )	return;

	if ( mode==PCL )	s = PCLDUPLEX ;
	else			s = PSDUPLEX ;

	if ( strstr(zopts, "duplex")!=NULL ) {
		if ( writecn(STDOUT,s,strlen(s)) != strlen(s) )
			logerr(3,"Cannot send duplex string");
	}
}

char *getcrstr(s, i)
char *s;
int i;
{
	int k=0, m, start;
	char *s1=(char *)0, *s2, *ptr=s;

	while (*ptr) 
		if ( *ptr++ == '\r') 	k++ ;	

	if ( i > k )	return(s);

	ptr = s ;
	start = 0;
	for ( m=0 ; m < strlen(s) ; m++ ) {
		if ( ptr[m] == '\015' ) {
			i-- ;
			if ( i == 0 )	return( &s[start] );
			else start = m + 2;
		}
	}

	return( s );
}

/*
 *
 */

int update_pfu(pfu_filename,fontfilename)
char *pfu_filename, *fontfilename;
{
  FILE *pfu;

  if ((pfu=fopen(pfu_filename,"w")) == NULL)
        logerr(1,"Cannot open/create pfu file : %s",pfu_filename);
  else
  {
        fprintf(pfu,"%s",fontfilename);
        fclose(pfu);
        log(4,"pfu updated successfully\n");
  }
};

int delete_pfu(pfu_filename)
char *pfu_filename;
{
   if (unlink(filename) != 0)
      logerr(2,"Cannot unlink pfu file <%s>, possibly already deleted",
             pfu_filename);
};

int newfont(pfuname,fontname)
char *pfuname, *fontname;
{
 int sendfont(), delete_pfu();

 deletefonts();
 if (sendfont(fontname) == 0)
    updatepfu=TRUE;
 delete_pfu(pfuname);
};

int sendfont(fontfilename)
char *fontfilename;
{
  FILE *ffp;
  int n,count=0;

  if (writecn(STDOUT, ASSIGN, strlen(ASSIGN)) != strlen(ASSIGN))
        logerr(3,"Cannot send ASSIGN font string");
  if ((ffp=fopen(fontfilename,"r")) == NULL)
  {
        logerr(3,"Cannot open file <%s>",fontfilename);
        fexit(FILTABORT);
  }

  while ((n = getc(ffp)) != EOF)
  {
        if (putc(n,stdout) != n)
           logerr(2,"ASCII <%s> not written to printer",n);
        else
           count++;
  }

  fflush(stdout);
  fclose(ffp);

  if (writecn(STDOUT, PGRESET, strlen(PGRESET)) != strlen(PGRESET))
        logerr(3,"Cannot send PGRESET font string");
  if (writecn(STDOUT, ASSIGN, strlen(ASSIGN)) != strlen(ASSIGN))
        logerr(3,"Cannot send ASSIGN string");
  if (writecn(STDOUT, MKPRIM, strlen(MKPRIM)) != strlen(MKPRIM))
        logerr(3,"Cannot send MKPRIM string");
  if (writecn(STDOUT, MKPERM, strlen(MKPERM)) != strlen(MKPERM))
        logerr(3,"Cannot send MKPERM string");

  return(0); /* success */
};


