/*
 ===========================================================================
 =                                                                         =
 =  (C) Copyright 1995, Computer Technology Institute (CTI)                =
 =                                                                         =
 =  Permission to use, copy, modify and distribute this software and       =
 =  its documentation for non-commercial use and without fee is hereby     =
 =  granted, provided that the above copyright notice appear in all        =
 =  copies and that both that copyright notice and this permission         =
 =  notice and warranty disclaimer appear in supporting documentation,     =
 =  and that the name of Computer Technology Institute (CTI) or any of     =
 =  its entities not be used in advertising or publicity pertaining to     =
 =  distribution of the software and its documentation without specific    =
 =  prior permission.                                                      =
 =                                                                         =
 =  Computer Technology Institute (CTI) disclaims all warranties with      =
 =  regard to this software and makes no representations about the         =
 =  suitability of this software and its documentation for any purpose.    =
 =  It is provided "as is" without expressed or implied warranty.          =
 =                                                                         =
 ===========================================================================
 =                                                                         =
 =                        Project CTI-Print                                =
 =                                                                         =
 = File:                                                                   =
 =   ifhpiii.c                                                             =
 =                                                                         =
 = Synopsis:                                                               =
 =   This is the main driver module which handles the printer interface.   =
 =                                                                         =
 = Author:                                                                 =
 =   Panos Dimakopoulos, Systems Programmer,                               =
 =   Computer Technology Institute,                                        =
 =   Division of Computing Facilities,                                     =
 =   P.O. Box 1122,                                                        =
 =   261 10  Patras,                                                       =
 =   Greece                                                                =
 =   (e-mail: dimakop@cti.gr)                                              =
 =   Tel: +30 61 992061                                                    =
 =   Fax: +30 61 993973                                                    =
 =                                                                         =
 ===========================================================================
*/


/****************************************************************************
 * Modification History:
 * 	Revision 1.1	94/10/01	21:31:22
 *		 Creation.
 * 	Revision 1.2	94/12/08	12:45:25
 *		 The count of nulls to be sent is specified automatically
 *		 from the MODEL env variable which can be set to III or IIID.
 * 	Revision 1.3	95/01/19	14:16:01
 *		 duplex and landscape capabilities added.
 * 	Revision 1.4	95/01/24	19:01:59
 *		 The sleep time before the resetprinter thought not to be
 *		 necessary. However this series has great problems with
 *		 the switching.
 * 	Revision 1.5	95/01/30	17:21:36
 *		 readline() owner's mention.
 * 	Revision 1.6	95/01/30	19:43:06
 *		 Level added to log functions.
 * 	Revision 1.7	95/02/10	18:35:06
 *		 CTI full address.
 * 	Revision 1.8	95/02/14	13:18:24
 *		 New Copyright notice.
 * 	Revision 1.9	95/05/30	18:05:17
 *		 Version 1.1 initiated.
 * 	Revision 1.10	95/08/23	18:10:51
 *		 Code added for selecting a different font from the
 *		 command line when submitting the job.
 *		 Also, file pfu in the spool area keeps the last
 *		 successfully downloaded font. If new font is the same
 *		 with that one then no downloading takes place.
 * 	Revision 1.11	95/08/24	16:48:13
 *		 Porting to Solaris initiated.
 * 	Revision 1.12	95/08/24	18:27:00
 *		 Warning messages about wait_printer() supressed.
 *
 */

#include <stdio.h>
#include <signal.h>
#include <stdlib.h>
#include <string.h>
#include <sys/termios.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/fcntl.h>
#include <unistd.h>
#include <hpiii.h>

static char idallf[] = "@(#) Computer Technology Institute, Project CTI-Print";
static char idhpiii[] = "@(#)       Filter for HP LaserJet III, V 1.2 \n" ;
static char idifhpiii[] = "@(#) ifhpiii.c 95/08/24 - V 1.12" ;
char    sendline[MAXLINE+1];

char *name;		/* name of filter */

int updatepfu = FALSE ;
char arr_fntname[150], *fntname = arr_fntname ;
char *pfu_fname = "pfu" ;	/* pfu is the file in the spool area that
				 * keeps the font path which has been
				 * successfully downloaded in the past.
				 */

/* 
 * The following variables are set by the arguments passed to the filter
 * from the caller (lpd)
 */
	char	*zopts, 
		*class, 
		*job, 
		*login, 
		*accntname, 
		*host, 
		*accntfile, 
		*format,
		*printer, 
		*filename, 
		*seq_number, 
		*username, 
		*date, 
		*pr_commnt;
	int	width, 
		length, 
		xwidth, 
		ylength,
		debug, 
		literal, 
		indent;

int prmsline;			/* line of printer_perms to change  */
int errorcode = FILTABORT;	/* value by which the errlog funcs exit */
int mode = 0;			/* PCL or Postscript */
int cartridge = OFF ;		/* Cartridge loaded or not */
int model;			/* printer model */
int npages=1, lines=0;		/* number of pages, number of lines */

main(argc,argv)
int argc;
char *argv[];
{
	int c, k, writecn();
	int lines = 0;
	char s[150];
	void cleanup();
	short wrap();
	short count_ch = 0;	/* Counts the chars in a single line in order to
				 * find the number of physical lines (when the
				 * end of line wrapping is enabled one long
				 * line can be printed as two on the page */

	debug = 2;
	getargs( argc, argv );

	get_retries();
	get_model();
	has_cartridge();

	/*
	 * Turn off SIGPIPE
	 */
	(void *)signal(SIGPIPE, SIG_IGN);
	(void *)signal(SIGINT, cleanup);
	(void *)signal(SIGTERM, cleanup);

	sendjob(stdin, STDOUT );

	if ( mode == PCL )	pclresetprinter();
	if ( mode == POSTSCRIPT )	eofprinter();

	updaccnt(npages);
	doaccnt();
	close(STDOUT);

	fexit(FILTSUCC);

}

int writecn(fd,ptr,nbytes)
register int fd;
register char *ptr;
int nbytes;
{
	int nleft, nwritten;
	void wait_printer();
	void (*f)();
	
	nleft = nbytes;
	while (nleft > 0) {
		f=signal(SIGALRM,wait_printer);
		alarm(3);
		nwritten = write (fd, ptr, nleft);
		alarm(0);
		signal(SIGALRM,f);
		if (nwritten == 0) {
			logerr(4,"writecn wrote nothing");
			continue ;
		}
		if (nwritten < 0) {
			logerr(4,"writecn failure");
			fexit(FILTRETRY);
		}
		nleft -= nwritten;
		ptr += nwritten;
	}
	return(nbytes - nleft);
}

void cleanup()
{
	writecn(STDOUT,"\003\004", strlen("\003\004"));  /* ctrl-C, EOF */
	writecn(STDOUT,PCLRESETSTR, strlen(PCLRESETSTR));
	log(4,"Signal received: Job Aborted before it finishes");
	fexit(FILTABORT);
}

static char null_str[]={"\000\000\000\000\000\000\000\000"};
#define NULL_LEN	8

setpcl_on()
{
	int cou, counulls=300, slptm=10;

	if (model == IIID)	{ counulls = 3000; slptm = 20; }
	if (model != IIISi)	sleep(10);
	if ( writecn(STDOUT,EOFSTR,strlen(EOFSTR)) != strlen(EOFSTR) )
		logerr(3,"Cannot send EOF");
	if (model != IIISi)	sleep(10);
	resetprinter();
	for (cou=0;cou<counulls;cou++)
		if ( writecn(STDOUT,null_str,NULL_LEN) != NULL_LEN )
			logerr_die(1,"Cannot send nulls");
	if ( writecn(STDOUT,PCLON,strlen(PCLON)) != strlen(PCLON) )
		logerr_die(1,"Cannot set PCL mode");
	sleep(slptm);
}
setps_on()
{
	int cou, counulls=300, slptm=10;

	if (model == IIID)	{ counulls = 3000; slptm = 20; }
	if (model != IIISi)	sleep(10);
	resetprinter();
	for (cou=0;cou<counulls;cou++)
		if ( writecn(STDOUT,null_str,NULL_LEN) != NULL_LEN )
			logerr_die(1,"Cannot send nulls");
	if ( writecn(STDOUT,POSTON,strlen(POSTON)) != strlen(POSTON) )
		logerr_die(1,"Cannot set PostScript mode");
	sleep(slptm);
}

eofprinter()
{
	if ( writecn(STDOUT,EOFSTR,strlen(EOFSTR)) != strlen(EOFSTR)  )
		logerr(3,"Cannot send EOF char");
}

resetprinter()
{
	if ( cartridge == OFF ) {
		pclresetprinter();
		return;
	}

	if ( writecn(STDOUT,UEL,strlen(UEL)) != strlen(UEL) )
		logerr(3,"Cannot UEL reset printer");
	if ( writecn(STDOUT,UEL,strlen(UEL)) != strlen(UEL) )
		logerr(3,"Cannot UEL reset printer");
}

pclresetprinter()
{
	if ( writecn(STDOUT,PCLRESETSTR,strlen(PCLRESETSTR)) != strlen(PCLRESETSTR) )
		logerr(3,"Cannot pcl reset printer");
}

deletefonts()
{
	if ( writecn(STDOUT,DELFONTS,strlen(DELFONTS)) != strlen(DELFONTS) )
		logerr(3,"Cannot delete fonts");
}

has_cartridge()
{
	char *s;

	s = getenv("CARTRIDGE") ;
	if ( ! s )	cartridge = OFF ;
	if ( ! strcmp(s,"ON") ) cartridge = ON;
		else {
			cartridge = OFF;
		}
}

/*
 * This function decides which font to download by reading 
 * MACHINESFONTS file.
 * If font to be sent already exists in the printer then no
 * downloading takes place.
 */
selectfont()
{
  FILE *pfu;    /* previous font used */
  char *s, *t, prevfnt[150], fulpathname[150];
  char *fnt_scan();
  int newfont();
  short resourcesave();

	if ( resourcesave() == ON ) update_pfu( pfu_fname, " " );
       /* find font for current job */
  if (zopts != NULL)
  {
        if ((s=strstr(zopts,"font")) != NULL)
        {
                log(1,"-Zfont option detected");
                t=strchr(s,'='); t++;
                log(1,"Step t=strchr()");
                if ((*t) == '/') {
                        sscanf(t,"%s",fntname);
			log(1,"font=[%s]",fntname);
                } else {
			sprintf(fntname,"%s/%s",FONTPATH,DEFLTFONT);
			logerr(1,"Only absolute paths accepted - Not: <%s>",t);
		}
        }
        else if ((s=fnt_scan(host)) == NULL)
           sprintf(fntname,"%s/%s",FONTPATH,DEFLTFONT);
        else if (strcmp(s,"NONE"))
           sprintf(fntname,"%s/%s",FONTPATH,s);
        else   /* when s="NONE" */
        {
           if (writecn(STDOUT, PGRESET, strlen(PGRESET)) != strlen(PGRESET))
             logerr(3,"Cannot send PGRESET string");
           return;
        };
  }
  else if ((s=fnt_scan(host)) == NULL)
        sprintf(fntname,"%s/%s",FONTPATH,DEFLTFONT);
  else if (strcmp(s,"NONE"))
        sprintf(fntname,"%s/%s",FONTPATH,s);
  else   /* when s="NONE" */
  {
        if (writecn(STDOUT, PGRESET, strlen(PGRESET)) != strlen(PGRESET))
            logerr(3,"Cannot send PGRESET string");
        return;
  };
        /* find font used on last job */
  sprintf(fulpathname,"%s",pfu_fname);
  fprintf(stderr,"fulpathname=[%s]\n",fulpathname); fflush(stderr);
  if ( (pfu=fopen(fulpathname,"r")) == NULL)
  {
        logerr(3,"Cannot open pfu file");
        newfont(fulpathname,fntname);
  }
  else if (fscanf(pfu,"%s\n",prevfnt) <= 0)
  {
        logerr(3,"Cannot read previously used font");
        newfont(fulpathname,fntname);
  }
  else    /* open and read successful */
  {
        fclose(pfu);

                /* if fonts different, send new font, else do nothing */
        if (strcmp(prevfnt,fntname) !=0 )
        {
           fprintf(stderr,"different fonts detected\n"); fflush(stderr);
           newfont(fulpathname,fntname);
        }
        else
        {
           log(3 ,"same fonts detected\n");
           if (writecn(STDOUT, SELFONT, strlen(SELFONT)) != strlen(SELFONT))
           {
                log(2,"Cannot send SELFONT string");
                newfont(fulpathname,fntname);
           }
           else
           {
                log(3,"Font selected successfully");
                updatepfu=FALSE;
           };
        }
  }
};


set_mode(m)
short m;
{
	if( m == POSTSCRIPT ) {
		if (cartridge==OFF) {
			errorcode = FILTABORT;
			writecn(STDOUT,UNABLE,strlen(UNABLE));
			pclresetprinter();
			fatal(1,"Cannot print Postscript without Cartridge");
		}
		mode = POSTSCRIPT ;
		log(3,"Entering POSTSCRIPT Mode");
		setps_on();
	} else {
		mode = PCL ;
		log(3,"Entering PCL Mode");
		if (cartridge==ON) setpcl_on();
			else pclresetprinter();
		page_set();
	}
}

page_set()
{
	char s[150], *ptr, lnd='0';


	if ( writecn(STDOUT,CRLFSTR,strlen(CRLFSTR)) != strlen(CRLFSTR) ) {
		logerr(3,"Cannot send CRLF string");
	}

	selectfont();

	if ( zopts ) {
		char *pptr = PGSTRDPL;
		if ( strstr(zopts, "landscape")!=NULL ) {
			lnd = '1' ;
			width = 132 ;
			length = 41 ;
			pptr = PGSTR;
			sprintf(s, PGSTR, lnd, width, length);
		}
		if ( strstr(zopts, "duplex")!=NULL ) {
			sprintf(s, pptr, lnd, width+5, length);
		}
	} else
		sprintf(s, PGSTR, lnd, width, length);

	if ( writecn(STDOUT,s,strlen(s)) != strlen(s) ) {
		logerr(3,"Cannot send Control Page string");
	}

	if ( width != 132 )	width = 80;

	if ( zopts && (strstr(zopts,"landscape")!=NULL) )	ptr = PITCHLND;
	else {
		if ( width==132 )	ptr = PITCH132;
		else			ptr = PITCH;
	}

	if ( writecn(STDOUT,ptr,strlen(ptr)) != strlen(ptr) )
		logerr(3,"Cannot send PITCH string");

	if ( zopts && (strstr(zopts,"duplex")!=NULL) ) 		ptr = SHIFTDPL;
	else 							ptr = SHIFT;

	if ( writecn(STDOUT,ptr,strlen(ptr)) != strlen(ptr) )
		logerr(3,"Cannot send SHIFT string");

	wrap_set();
}

/*
 * If line wrapping is required the WRAP env variable should be set
 * to the value ON the same way as the CCB_IIIFNT env var above.
 * Edit the /usr/lib/hp/filters/grifilter.sh script accordingly.
 */
wrap_set()
{
	short wrap();

	if ( wrap() == ON )
		if ( writecn(STDOUT,WRAPON,strlen(WRAPON)) != strlen(WRAPON) )
			logerr(3,"Cannot send WRAPON string");
}

short wrap()
{
	char *s;

	s = getenv("WRAP") ;
	if ( ! s )			return ( OFF );
	if ( ! strcmp(s,"ON") )		return ( ON );
		else			return ( OFF );
}

open_trace()
{
	char *s;
	int fd;

	s = getenv("TRACE");
	if ( s ) {
		if (!strcmp(s,"ON")) {
			char ss[50];

			sprintf(ss,"/tmp/hpiii.%04d", getpid());
			fd = open(ss, O_WRONLY|O_CREAT,0600);
			if (fd < 0) {
				log(2,"Cannot open trace file");
				return( 0 );
			} else	return( fd );
		} else	return ( 0 );
	} else
		return (0);
}

sendjob(fp, sockfd)
register FILE    *fp;
register int    sockfd;
{
	fd_set writefds;
	short rdcnt, wrcnt;
	char *tmp;
	struct timeval timeout;
	int tracefd;
	void wait_printer();


	tracefd = open_trace();

	timeout.tv_sec = 0;
	timeout.tv_usec = 0;

    do {
        FD_ZERO(&writefds);
        FD_SET(sockfd, &writefds);
        select(sockfd+1, (fd_set *) 0, &writefds, (fd_set *) 0, &timeout);
        if (FD_ISSET(sockfd, &writefds)) {
		if (rdcnt=readline(fileno(fp), sendline, MAXLINE)) {
		if (tracefd)	write(tracefd, sendline, rdcnt);
		tmp = sendline;
                if (!mode) {
                    if (!strncmp("%!",sendline,2) || !strncmp("\004%!",sendline,2)) {
				if ( sendline[0] == '\004' ) 
					{tmp++ ; rdcnt-- ; }
				set_mode(POSTSCRIPT);
			} else set_mode(PCL);
		pr_duplex();
                }
                wrcnt=writecn(sockfd, tmp, rdcnt);
		if (wrcnt != rdcnt)
			logerr_die(2,"Cannot write what read (%d bytes)",rdcnt);
            }
        } else	wait_printer();		/* Check if printer went off line */

    } while (rdcnt != 0);
	close(tracefd);
}

/*
 * The following function has been copied from a printer driver 
 * written by Lawrence LeMay (lemay@epx.cis.umn.edu).
 */
/*
 * Read a line from a descriptor.  Read the line one byte at a time,
 * looking for a newline or '\014'.  We store the newline in the buffer,
 * then follow it with a null (the same as fgets(3)).
 * We return the number of characters up to, but not including,
 * the null (the same as strlen(3)).
 */

int
readline(fd, ptr, maxlen)
register int    fd;
register char   *ptr;
register int    maxlen;
{
    int                      n, rc;
    char                     c;
    static struct timeval    timeout;
    fd_set                   readfds;
    char                     *head;

    head = ptr;
    timeout.tv_sec = 1;
    timeout.tv_usec = 0;

    for (n = 0; n < maxlen; n++) {
        FD_ZERO(&readfds);
        FD_SET(fd, &readfds);
        if (select(fd+1, &readfds, (fd_set *) 0, (fd_set *) 0,
                &timeout) < 0) {
            logerr(1,"hpdriver: select error");
            fexit(1);
        }

        if (!FD_ISSET(fd, &readfds)) break;

        if ( (rc = read(fd, &c, 1)) == 1) {
		*ptr++ = c;
		if (c == '\n') {
			n++ ;
			++lines;
			if( lines > length ){
				lines -= length;
				++npages;
			}
			break;
		}
        } else if (rc == 0) {
            if (n == 0)
                return(0);    /* EOF, no data read */
            else
                break;        /* EOF, some data was read */
        } else
            return(-1);    /* error */
    }

    *ptr = 0;
    return(n);
}

/*
 *  doaccnt()
 *  writes the accounting information to the accounting file
 *  This has the format: user host printer pages format date
 */
doaccnt()
{
	time_t t, time();
	char *ctime();
	FILE *f;

	t = time((time_t *)0);
	if(accntfile && access(accntfile, W_OK) >= 0 &&
	    (f = fopen(accntfile, "a" )) != NULL) {
		fprintf(f,"%s\t%s\t%s\t%s\t%7d\t%s\t%s",
			login? login: "NULL", 
			host? host: "NULL", 
			printer? printer: "NULL", 
			filename? filename: "NULL",
			npages,
			format? format: "NULL", 
			seq_number ? seq_number : "XXX",
			ctime(&t));
		(void)fclose(f);
	}
}


get_model()
{
	char *s;

	s = getenv("MODEL");
	if (s) {
		if ( ! strcmp(s, "III") )	model = III ;
		if ( ! strcmp(s, "IIID") )	model = IIID ;
		if ( ! strcmp(s, "IIISi") )	model = IIISi ;
	}
}


#define PSDUPLEX "statusdict begin true setduplexmode end\r\n"
#define PCLDUPLEX "\033&l1S"

pr_duplex()
{
	char *s;

	if ( ! zopts )	return;

	if ( mode==PCL )	s = PCLDUPLEX ;
	else			s = PSDUPLEX ;

	if ( strstr(zopts, "duplex")!=NULL ) {
		if ( writecn(STDOUT,s,strlen(s)) != strlen(s) )
			logerr(3,"Cannot send duplex string");
	}
}

int update_pfu(pfu_filename,fontfilename)
char *pfu_filename, *fontfilename;
{
  FILE *pfu;

  if ((pfu=fopen(pfu_filename,"w")) == NULL)
        logerr(1,"Cannot open/create pfu file : %s",pfu_filename);
  else
  {
        fprintf(pfu,"%s",fontfilename);
        fclose(pfu);
        log(4,"pfu updated successfully\n");
  }
};

int delete_pfu(pfu_filename)
char *pfu_filename;
{
   if (unlink(filename) != 0)
      logerr(2,"Cannot unlink pfu file <%s>, possibly already deleted",
             pfu_filename);
};

int newfont(pfuname,fontname)
char *pfuname, *fontname;
{
 int sendfont(), delete_pfu();

 deletefonts();
 if (sendfont(fontname) == 0)
    updatepfu=TRUE;
 delete_pfu(pfuname);
};

int sendfont(fontfilename)
char *fontfilename;
{
  FILE *ffp;
  int n,count=0;

  if (writecn(STDOUT, ASSIGN, strlen(ASSIGN)) != strlen(ASSIGN))
        logerr(3,"Cannot send ASSIGN font string");
  if ((ffp=fopen(fontfilename,"r")) == NULL)
  {
        logerr(3,"Cannot open file <%s>",fontfilename);
        fexit(FILTABORT);
  }

  while ((n = getc(ffp)) != EOF)
  {
        if (putc(n,stdout) != n)
           logerr(2,"ASCII <%s> not written to printer",n);
        else
           count++;
  }

  fflush(stdout);
  fclose(ffp);

  if (writecn(STDOUT, PGRESET, strlen(PGRESET)) != strlen(PGRESET))
        logerr(3,"Cannot send PGRESET font string");
  if (writecn(STDOUT, ASSIGN, strlen(ASSIGN)) != strlen(ASSIGN))
        logerr(3,"Cannot send ASSIGN string");
  if (writecn(STDOUT, MKPRIM, strlen(MKPRIM)) != strlen(MKPRIM))
        logerr(3,"Cannot send MKPRIM string");
  if (writecn(STDOUT, MKPERM, strlen(MKPERM)) != strlen(MKPERM))
        logerr(3,"Cannot send MKPERM string");

  return(0); /* success */
};


/*
 * If NOTRESRCSAVE env var in ifhpiii.sh is set to ON then font is
 * always downloaded.
 */
short resourcesave()
{
	char *s;

	s = getenv("NOTRESRCSAVE") ;
	if ( ! s )			return ( OFF );
	if ( ! strcmp(s,"ON") )		return ( ON );
		else			return ( OFF );
}

