/*==========================================================================
 =                                                                         =
 =                        Project CTI-Print                                =
 =                                                                         =
 = Author:                                                                 =
 =   Panos Dimakopoulos, Systems Programmer,                               =
 =   Computer Technology Institute,                                        =
 =   Division of Computing Facilities,                                     =
 =   P.O. Box 1122,                                                        =
 =   261 10  Patras,                                                       =
 =   Greece                                                                =
 =   (e-mail: dimakop@cti.gr)                                              =
 =   Tel: +30 61 992061                                                    =
 =   Fax: +30 61 993973                                                    =
 =                                                                         =
 = Created by Patrick Powell  <papowell@sdsu.edu>                          =
 =   for LPRng software Sat Aug 26 06:54:25 PDT 1995                       =
 ==========================================================================*/


/*==========================================================================
  Version CTI-LPRng-1.0
  =========================================================================*/


/*
 * $Id: pr_status.c,v 1.2 1995/09/04 14:58:10 papowell Exp $
 */

#include "portable.h"
#include "common.h"
#include "hp4.h"


/*
 * Note: this routine is used for explicitly asking for status,
 * and only status.  It is assumed that the pr_sync() routine
 * will be called first,  then the timed status and other status
 * turned off.  Unfortunately,  there can be a large amount of
 * status information pending at this point, and we have to clean it out.
 */

int pr_status(fd)
int fd;
{
	int st=0, status = -1, fftries = 0;
	int i = 0;

	log(2,"Asking for printer status");

	query[i++]=INFOSTATUS;

	signal(SIGALRM,pr_late);
	alrmflag=0;
	alarm(120);

	do {
		log(4, "pr_status: fftries=%d", ++fftries);
		if( status < 0 ){
			/* we put a 3 second timeout on this */
			pr_query(fd, i);
			readpipe(&st,3);
			if( st > 0 ){
				status = st;
			}
		} else {
			/* get any other pending status information */
			readpipe(&st,1);
			if( st > 0 ){
				status = st;
			} else {
				break;
			}
		}
	} while( !alrmflag );

	alarm(0);
	signal(SIGALRM,SIG_DFL);
	return( status );
}
