/*==========================================================================
 =                                                                         =
 =                        Project CTI-Print                                =
 =                                                                         =
 = Author:                                                                 =
 =   Panos Dimakopoulos, Systems Programmer,                               =
 =   Computer Technology Institute,                                        =
 =   Division of Computing Facilities,                                     =
 =   P.O. Box 1122,                                                        =
 =   261 10  Patras,                                                       =
 =   Greece                                                                =
 =   (e-mail: dimakop@cti.gr)                                              =
 =   Tel: +30 61 992061                                                    =
 =   Fax: +30 61 993973                                                    =
 =                                                                         =
 = Created by Patrick Powell  <papowell@sdsu.edu>                          =
 =   for LPRng software Sat Aug 26 06:54:25 PDT 1995                       =
 ==========================================================================*/


/*==========================================================================
  Version CTI-LPRng-1.0
  =========================================================================*/


/*
 * $Id: pr_synch.c,v 1.4 1995/09/10 14:43:29 papowell Exp $
 */

#include "portable.h"
#include "common.h"
#include "hp4.h"


/*
 * In the PJL manual, Chapter 9, Section: Determining if printer status
 * readback is available it states that if the application does not
 * receive a response within three seconds, then the application can 
 * assume printer status readback is currently unavailable.
 */

int pr_synch(fd, timeout)
int fd;
int timeout;
{
	int   synchronised = 0, fftries = 0;
	char buf[MAXLINE+1];
	char sync[MAXLINE+1];
	char timed[MAXLINE+1];
	void (*f)();
	int len;
	int i = 0;

	if( timeout == 0 ) timeout = 60;
	log(2,"Synchronisation attempt, %d sec timeout", timeout);
	

	query[i++]=UEL;
	query[i++]=PCLRESETSTR;
	query[i++]=UELPJL;
	query[i++]=USTPJLOFF;
	query[i++]=USTPJLDEV;
	query[i++]=USTPJLJOB;
	if( wait_time ){
		(void)plp_snprintf(timed,sizeof(timed)-1,USTPJLTIMED, wait_time);
		query[i++]=timed;
	}
	(void)plp_snprintf(sync,sizeof(sync)-1,"Seq %d",getpid());
	(void)plp_snprintf(buf,sizeof(buf)-1,"@PJL ECHO %s\r\n",sync);
	query[i++]=buf;

	f=signal(SIGALRM,pr_late);
	do{
		alrmflag=0;
		alarm(timeout);
		do {
			if( !synchronised) pr_query(fd, i);
			log(4, "pr_synch: tries=%d", ++fftries);
			len = readpipe((int *)0,3);
			if( len <= 0 ) continue;
			if( strstr(readprin,sync) ) {
				synchronised = TRUE;
			} else if( strstr(readprin, "@PJL ECHO") ) {
				log(2,"pr_synch: synchro from previous attempt '%s'", readprin);
			}
		} while( !synchronised && !alrmflag );
		if( ! synchronised ){
			log(0,"No response from printer for %d secs; offline?", timeout);
		}
	} while( !synchronised );
	alarm(0);
	signal(SIGALRM,f);
	pr_startjob(STDOUT);
	return( synchronised );
}

void header_info()
{
	int i = 0;
	query[i++]=UEL;
	query[i++]=PCLRESETSTR;
	query[i++]=UELPJL;
	pr_query(STDOUT, i);
}
