/*==========================================================================
 =                                                                         =
 =                        Project CTI-Print                                =
 =                                                                         =
 = Author:                                                                 =
 =   Panos Dimakopoulos, Systems Programmer,                               =
 =   Computer Technology Institute,                                        =
 =   Division of Computing Facilities,                                     =
 =   P.O. Box 1122,                                                        =
 =   261 10  Patras,                                                       =
 =   Greece                                                                =
 =   (e-mail: dimakop@cti.gr)                                              =
 =   Tel: +30 61 992061                                                    =
 =   Fax: +30 61 993973                                                    =
 =                                                                         =
 = Created by Patrick Powell  <papowell@sdsu.edu>                          =
 =   for LPRng software Sat Aug 26 06:54:25 PDT 1995                       =
 ==========================================================================*/


/*==========================================================================
  Version CTI-LPRng-1.0
  =========================================================================*/


/*
 * $Id: sendjob.c,v 1.4 1995/10/27 13:42:50 papowell Exp $
 */

#include "portable.h"
#include "common.h"
#include "hp4.h"

/*
 * send job to the remote printer
 */

void sendjob(fp, sockfd)
FILE    *fp;
int    sockfd;
{
	int 	rdcnt;
	int 	tracefd;
	int i;
	fd_set	writefds, initwritefds;
	fd_set	readfds, initreadfds;
	struct timeval timeout;
	int nfd;
	char *s, *ps;
	int discard = 0;

	tracefd = open_trace();

	log(4,"sendjob: starting");
	FD_ZERO(&initwritefds);
	FD_SET(sockfd, &initwritefds);
	FD_ZERO(&initreadfds);
	nfd = sockfd+1;
	if( pipefd[0] ){
		FD_SET(pipefd[0], &initreadfds);
		if( nfd <= pipefd[0] ) nfd = pipefd[0]+1;
	}
	do {
		/* check to see if the printer went off line */
		/* WARNING: this is actually a bogus check! The system will
		 * put the output characters in a queue and may or may not
		 * return when output has stopped.  However, it is better than
		 * doing nothing.  A bombproof version would do character at
		 * a time IO (ack! hack!) which would have, shall we say,
		 * 'significant effects on the system overhead'
		 * Patrick Powell Sun Aug 27 07:51:59 PDT 1995
		 */
		do {
			timeout.tv_sec = 0;
			timeout.tv_usec = 0;
			readfds = initreadfds;
			writefds = initwritefds;
			i = select(nfd, &readfds, &writefds,
				(fd_set *) 0, &timeout);
			log(8,"sendjob: fd %d select status %d", sockfd, i );
			if( i == 0 ){
				wait_printer(FILTRETRY); /* Check if printer went off line */
			}
		} while( i <= 0 );
		if( FD_ISSET( pipefd[0], & readfds ) ){
			log(4,"sendjob: pending status from printer");
			readpipe((int *)0,3);
		}
		if( !FD_ISSET( sockfd, &writefds ) ){
			continue;
		}
		/* read an input line */
		log(9,"sendjob: reading input");
		if( (rdcnt=file_readline(fileno(fp),sendline,sizeof(sendline)-1)) > 0){
			sendline[rdcnt] = 0;
			log(6,"sendjob: read %d '%s'", rdcnt,sendline);
			if (tracefd > 0) write(tracefd, sendline, rdcnt);
			/* try to determine which mode you are in */
			if (!mode) {
				if( literal || autodetect ){
					log(2,"Entering Autodetect Mode");
					mode = LITERAL;
				} else {
					if( (ps = strstr(sendline, "%!" )) ){
						for( s = sendline; ps && s!=ps; ++s ){
							/* we must have a control char string */
							if( !iscntrl(*s) ) ps = 0;
						}
					}
					if( ps && cartridge ){
						mode = POSTSCRIPT ;
						log(2,"Entering POSTSCRIPT Mode");
						setps_on();
					} else {
						mode = PCL ;
						log(2,"Entering PCL Mode");
						setpcl_on();
						page_set();
						if( ps ){
							s = UNABLE;
							i = strlen(s);
							writecn(sockfd,s,i);
							discard = 1;
						}
					}
				}
				log(4,"sendjob: setting duplex");
				pr_duplex();
			}	/* if (!mode) */
			if( !discard && rdcnt != writecn(sockfd, sendline, rdcnt) ){
				logerr_die(2,"Cannot write what read (%d bytes)",rdcnt);
			}
			log(9,"sendjob: wrote line");
   		}  /* if file_readline */
    } while ( rdcnt > 0 );
	close(tracefd);
	log(4,"sendjob: ending");
}
