/****************************************************************************
 *			p s f c e n d b a n n e r . c		            *
 *									    *
 *	Print an end banner page on a color postscript printer		    *
 *									    *
 * The generated print stream is NOT minimally conforming as per Red Book.  *
 *									    *
 *	Tony Field.       tony@ajfcal.cuc.ab.ca				    *
 ****************************************************************************
*/

/*
 * $Id: psfcendbanner.c,v 3.4 1992/02/24 06:48:59 ajf Exp ajf $
 * Modified for use with PLP by Stefano Ianigro (stef@unibw-hamburg.de)
 * Sat Mar 18, 21:22, 1995
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ctp.h"
#include "psf.h"
#include "patchlevel.h"

/*
 * modify the following font selections if language changes are needed in
 * psfprint.def. Also enable the language translation code at line 257  (
 * if (need_xlate)  )
 */

#define FONT1 "/Bookman-Demi findfont %d scalefont setfont\n" 
#define FONT2 "/Helvetica-Bold set_font /Helvetica-Bold findfont %d scalefont setfont\n"
#define FONT3 "/Courier set_font /Courier findfont %d scalefont setfont\n"
#define FONT4 "/Helvetica-Bold set_font /Helvetica-Bold findfont %d scalefont setfont\n"
#define FONT5 "/Times-Bold findfont 40 scalefont setfont\n"

/* setup paper dimensions */

typedef struct			/* measurement in point  */
  {
    char paper_name[60];	/* name of paper size (for command
				 * line)       
				 */
    char paper_tray[200];	/* postscript operator to
				 * select this tray    
				 */
    int width;			/* portrait point width of paper   */
    int height;			/* portrait point height of paper  */
    int lx, ly, ux, uy;
  }
measure;

measure *p;			/* working set of parameters  (points to
				 * something below)    
				 */

/* standard paper sizes in portrait layout.                        */

measure page_types[NPAGE] =
{
  {"letter",
   "statusdict begin lettertray end",
   612, 792, 18, 15, 593, 777
  },
  {"legal",
   "statusdict begin legaltray end",
   612, 1008, 18, 15, 593, 993
  },
  {"a4",
   "statusdict begin a4tray end",
   595, 842, 18, 15, 578, 827
  },
  {"b5",
   "statusdict begin b5tray end",
   516, 729, 18, 15, 497, 712
  },
  {"",				/* end of paper list */
   "",
   0, 0, 0, 0, 0, 0
  }
};

char slots[NSLOTS][600] =
{
  "statusdict begin 1 setpapertray end",
  "statusdict begin 2 setpapertray end",
  "", "", "", ""
};

int nslots = 5;

int X_INDENT = 30;		/* points                                        */
int Y_INDENT = 30;
int BIG_POINT = 60;		/* changes to point size automatically scale     */
int MED_POINT = 35;		/* the printout.                                 */
int SMALL_POINT = 15;
int UX, UY, DX, DY, MAX_X, MAX_Y;
int is_qms2425 = 0;
/*
 * The following arguments are received in the command line argv[9] and
 * argv[10] may be page size and paper tray specifications. these could be:
 * 
 * -b n = select paper bin n -g size = selelct paper tray (letter, b5, etc)
 */
#ifdef USE_WITH_PLP
char Division[200];	/* division name   */
char Printer[64];	/* printer name    */
char Title[64];  	/* jobname         */
char Options[64];	/* Z-options       */
char Requestid[10]; 	/* Sequence number */
char Date[30];   	/* Date            */
char Name[64];  	/* real username   */
char Userid[9]; 	/* login name      */
char Machineid[64];	/* host            */
char User[9];   	/* login name      */
char Class[2];          /* Class           */
char buf[200];
#else /* NOT to use with PLP */
#define	Userid		argv[1]
#define	Name		argv[2]
#define	Requestid	argv[3]
#define	Printer		argv[4]
#define	Options		argv[5]
#define	Date		argv[6]
#define	Machineid	argv[7]
#define	Title		argv[8]
#endif

char *defref = NULL;

FILE *outfp;
unsigned char xlate[256];	/* translation vector    */
int need_xlate = 1;		/* default xlate      */
FILE *pdef;
void trim ();

main (argc, argv)
     int argc;
     char *argv[];
{
  int x, y, c, i, set_paper_tray, set_paper_bin, number, j, k, l;
  char line[200];

  if (argc < 8) {
    usage ();
  }

  for (i = 0; i < 256; i++)
    xlate[i] = i;

  outfp = stdout;
  pdef = NULL;

/* get user specified def file with -u */

  for (i = 0; i < argc; i++)
    {
      if (strncmp (argv[i], "-u", 2) == 0)
	{
	  if (strlen (argv[i]) > 2)
	    defref = argv[i] + 2;
	  else
	    defref = argv[i + 1];
	  break;
	}
    }
  if (i >= argc)
    defref = PDEF;

  if ((pdef = fopen (defref, "r")) != NULL)
    {
      char line_type[200];

      while (fgets (line, 200, pdef))
	{
	  if (strncmp (line, "*printer", 8) == 0)
	    break;
	}
      nslots = 0;
      fgets (line, 200, pdef);	/* skip printer name     */
      if ((strstr (line, "QMS")) && (strstr (line, "2425"))) {
	  is_qms2425 = 1;
      }
      while (fgets (line, 200, pdef))
	{
	  trim (line);
	  sscanf (line, "%s%d", line_type, &number);

	  if (strcmp (line_type, "*paper") == 0)
	    {
	      for (i = j = 0; i < number; i++)
		{
		  if (i >= NPAGE - 1)
		    {
		      fgets (line, 200, pdef);
		      fgets (line, 200, pdef);
		      fgets (line, 200, pdef);
		      continue;
		    }
		  if (fgets (line, 200, pdef) == NULL)
		    bad_file ();
		  line[59] = 0;
		  trim (line);
		  strcpy (page_types[i].paper_name, line);
		  if (fgets (line, 99, pdef) == NULL)
		    bad_file ();
		  line[99] = 0;
		  trim (line);
		  strcpy (page_types[i].paper_tray, line);
		  if (fgets (line, 100, pdef) == NULL)
		    bad_file ();
		  trim (line);
		  sscanf (line, "%d%d%d%d%d%d",
			  &page_types[i].width, &page_types[i].height,
			  &page_types[i].lx, &page_types[i].ly,
			  &page_types[i].ux, &page_types[i].uy);
		  j = i;
		}
	      page_types[j + 1].paper_name[0] = '\0';
	    }
	  else if (strcmp (line_type, "*slots") == 0)
	    {
	      for (j = 0; j < NSLOTS; j++)
		slots[j][0] = 0;
	      for (i = j = 0; i < number; i++)
		{
		  if (is_qms2425) {
		      k = 3;
		      for (l = 0; l < k; l++) {
			  if (fgets (line, 200, pdef) == NULL)
			      bad_file ();
			  if (i >= NSLOTS)
			      continue;
			  /*	  line[99] = 0; */
			  trim (line);
			  if (l == 0) {
			      strcpy (slots[i], line);
			      j = i;
			  } else {
			      strcat (slots[i], "\n");
			      strcat (slots[i], line);
			  }
		      }
		  } else {
		      if (fgets (line, 200, pdef) == NULL)
			  bad_file ();
		      if (i >= NSLOTS)
			  continue;
		      /* line[99] = 0; */
		      trim (line);
		      strcpy (slots[i], line);
		      j = i;
		  }
		}
	      nslots = j + 1;
	    }
	  else if (strcmp (line_type, "*eof") == 0)
	    break;
	}
    }
  MAX_X = page_types[0].width;
  MAX_Y = page_types[0].height;
  set_paper_bin = 0;
  set_paper_tray = -1;

#ifdef USE_WITH_PLP
  for (i = 0; i < argc; i++)
#else
  for (i = 9; i < argc; i++)
#endif
    {
      if (argv[i][0] != '-')
	continue;
      c = argv[i][1];
      switch (c)
	{
	case 'b':
	  if ((set_paper_bin = atoi (argv[i + 1])) > nslots)
	    {
	      fprintf (stderr, "paper bin greater than %d\n", nslots);
	      exit (DISCARD_JOB);
	    }
	  i++;
	  break;

	case 'g':
	  for (set_paper_tray = 0; page_types[set_paper_tray].paper_name[0]; set_paper_tray++)
	    {
	      if (compare (argv[i + 1], page_types[set_paper_tray].paper_name) == 0)
		{
		  MAX_X = page_types[set_paper_tray].width;
		  MAX_Y = page_types[set_paper_tray].height;
		  break;
		}
	    }
	  if (page_types[set_paper_tray].paper_name[0] == '\0')
	    {
	      fprintf (stderr, "Invalid paper size\n");
	      exit (DISCARD_JOB);
	    }
	  i++;
	  break;

	case 'u':		/* already processed */
	  break;
#ifdef USE_WITH_PLP
	case 'B':               /* Division name */
	    (void) sprintf (Division, "%s", argv[i]+2);
	  break;
	case 'J':               /* Job name */
	    (void) sprintf (Title, "%s", argv[i]+2);
	  break;
	case 'S':               /* Printer name */
	    (void) sprintf (Printer, "%s", argv[i]+2);
	  break;
	case 'j':
	case 'q':               /* Sequence number */
	    (void) sprintf (Requestid, "%s", argv[i]+2);
	  break;
	case 'C':               /* Class */
	    (void) sprintf (Class, "%s", argv[i]+2);
	  break;
	case 't':
	case 'D':               /* Date */
	    (void) sprintf (Date, "%s", argv[i]+2);
	  break;
	case 'n':               /* userid */
	    (void) sprintf (Userid, "%s",  argv[i]+2);
	    (void) sprintf (User, "%s",  argv[i]+2);
	  break;
	case 'h':               /* hostname */
	    (void) sprintf (Machineid, "%s", argv[i]+2);
	  break;
	case 'Z':               /* Z-options */
	    (void) sprintf (Options, "%s", argv[i]+2);
	  break;
	case 'L':               /* Real user name */
	    (void) sprintf (Name, "%s", argv[i]+2);
	  break;
#endif
	default:;
	}
    }

  UX = X_INDENT;
  UY = (MAX_Y - Y_INDENT);
  DX = (MAX_X - 2 * X_INDENT -25);
  DY = (BIG_POINT * 3);

/* center text used for userid and job title print  */

  send ("\n");
  send ("%!\n");
  send ("statusdict /setduplexmode known\n");
  send ("{ statusdict begin false setduplexmode end } if\n");
  send ("statusdict /settumble known\n");
  send (" { statusdict begin false settumble end } if\n");
  send ("/ISOfonts 64 dict def\n"); /* Prologue for ISO fonts */
  send ("/ISOEncoding 256 array dup 0 StandardEncoding putinterval\n");
  send ("/#copies 1 def\n");
  send ("dup 8#244 /currency put\n");
  send ("dup 8#250 /dieresis put\n");
  send ("dup 8#252 /ordfeminine put\n");
  send ("dup 8#260 /ring put\n");
  send ("dup 8#272 /ordmasculine put\n");
  send ("dup 8#300 /Agrave put\n");
  send ("dup 8#301 /Aacute put\n");
  send ("dup 8#302 /Acircumflex put\n");
  send ("dup 8#303 /Atilde put\n");
  send ("dup 8#304 /Adieresis put\n");
  send ("dup 8#305 /Aring put\n");
  send ("dup 8#306 /AE put\n");
  send ("dup 8#307 /Ccedilla put\n");
  send ("dup 8#310 /Egrave put\n");
  send ("dup 8#311 /Eacute put\n");
  send ("dup 8#312 /Ecircumflex put\n");
  send ("dup 8#313 /Edieresis put\n");
  send ("dup 8#314 /Igrave put\n");
  send ("dup 8#315 /Iacute put\n");
  send ("dup 8#316 /Icircumflex put\n");
  send ("dup 8#317 /Idieresis put\n");
  send ("dup 8#321 /Ntilde put\n");
  send ("dup 8#322 /Ograve put\n");
  send ("dup 8#323 /Oacute put\n");
  send ("dup 8#324 /Ocircumflex put\n");
  send ("dup 8#325 /Otilde put\n");
  send ("dup 8#326 /Odieresis put\n");
  send ("dup 8#330 /Oslash put\n");
  send ("dup 8#331 /Ugrave put\n");
  send ("dup 8#332 /Uacute put\n");
  send ("dup 8#333 /Ucircumflex put\n");
  send ("dup 8#334 /Udieresis put\n");
  send ("dup 8#337 /germandbls put\n");
  send ("dup 8#340 /agrave put\n");
  send ("dup 8#341 /aacute put\n");
  send ("dup 8#342 /acircumflex put\n");
  send ("dup 8#343 /atilde put\n");
  send ("dup 8#344 /adieresis put\n");
  send ("dup 8#345 /aring put\n");
  send ("dup 8#346 /ae put\n");
  send ("dup 8#347 /ccedilla put\n");
  send ("dup 8#350 /egrave put\n");
  send ("dup 8#351 /eacute put\n");
  send ("dup 8#352 /ecircumflex put\n");
  send ("dup 8#353 /edieresis put\n");
  send ("dup 8#354 /igrave put\n");
  send ("dup 8#355 /iacute put\n");
  send ("dup 8#356 /icircumflex put\n");
  send ("dup 8#357 /idieresis put\n");
  send ("dup 8#361 /ntilde put\n");
  send ("dup 8#362 /ograve put\n");
  send ("dup 8#363 /oacute put\n");
  send ("dup 8#364 /ocircumflex put\n");
  send ("dup 8#365 /otilde put\n");
  send ("dup 8#366 /odieresis put\n");
  send ("dup 8#370 /oslash put\n");
  send ("dup 8#371 /ugrave put\n");
  send ("dup 8#372 /uacute put\n");
  send ("dup 8#373 /ucircumflex put\n");
  send ("dup 8#374 /udieresis put\n");
  send ("dup 8#377 /ydieresis put\n");
  send ("\n");
  send ("def\n");
  send ("/srgb {setrgbcolor} bind def\n");
  send ("/col0 {0.000 0.000 0.000 srgb} bind def\n");
  send ("/col4 {1.000 0.000 0.000 srgb} bind def\n");
  send ("/col1 {0.000 0.000 1.000 srgb} bind def\n");
  send ("\n");
  send ("ISOfonts /Symbol dup findfont put\n");
  send ("/ISOfont\n");
  send ("{\n");
  send (" dup ISOfonts exch known not\n");
  send (" {\n");
  send ("  ISOfonts begin\n");
  send ("  dup dup findfont\n");
  send ("  dup length 5 add dict /newdict exch def\n");
  send ("  {1 index /FID ne\n");
  send ("   {newdict 3 1 roll put}\n");
  send ("   {pop pop}\n");
  send ("   ifelse\n");
  send ("  }forall\n");
  send ("  newdict /Encoding ISOEncoding put\n");
  send ("  dup newdict definefont def\n");
  send ("  end\n");
  send (" }if\n");
  send (" ISOfonts exch get\n");
  send ("}def\n");
  send ("\n");
  send ("/set_font{ISOfont setfont}def\n");
  send ("\n");
  send ("/ctextu     % center text:   string x y dx\n");
  send ("{\n");
  send ("   2 div\n");
  send ("   /Dx exch def /Yv exch def /Xv exch def\n");
  send ("   dup stringwidth pop\n");
  send ("   2 div Dx exch sub Xv add Yv\n");
  send ("   gsave translate 0 0 moveto\n");
  send ("    {\n");
  send ("       /charcode exch def\n");
  send ("       /thechar (-) dup 0 charcode put def\n");
  send ("       thechar true charpath stroke\n");
  send ("       thechar stringwidth pop 0 translate 0 0 moveto\n");
  send ("    } forall\n");
  send ("    grestore\n");
  send ("} def\n");
  send ("/ctext {     % center text:   string x y dx\n");
  send ("   2 div\n");
  send ("   /Dx exch def /Yv exch def /Xv exch def\n");
  send ("   dup stringwidth pop\n");
  send ("   2 div Dx exch sub Xv add Yv moveto\n");
  send ("   show\n");
  send ("} def\n");
  replacement_fonts ();		/* get font translations */
  send ("%%EndProlog\n");

/*
 * Uncomment the following code if languange translation is needed for the
 * following parameters
 */

  if (need_xlate)
    {
#ifdef USE_WITH_PLP
      xlate_string (Userid);
      xlate_string (Name);
      xlate_string (Requestid);
      xlate_string (Title);
#else
      xlate_string (Userid);
      xlate_string (Name);
      xlate_string (Requestid);
      xlate_string (Title);
#endif
    }
  send ("%%Page: ? 1\n");
  if (set_paper_bin)
    printf ("%s\n", slots[set_paper_bin - 1]);

  if (set_paper_tray >= 0)
    printf ("%s\n", page_types[set_paper_tray].paper_tray);

  send ("/pg save def\n");

/* draw a box for the userid */

  printf ("gsave 0.7 setgray\n");
  printf ("newpath\n");
  printf ("%d %d moveto\n", UX, UY);
  printf ("%d %d rlineto\n", DX, 0);
  printf ("%d %d rlineto\n", 0, -DY);
  printf ("%d %d rlineto\n", -DX, 0);
  printf ("closepath\n");
  printf ("4 setlinewidth\n");
  printf ("stroke\n");

/* center the userid and the job title */
  y = UY - (DY + MED_POINT * 3);
  printf (FONT2, MED_POINT);
  printf ("(%s) %d %d %d ctext grestore\n", Title, X_INDENT, y, DX);


  y = UY - DY / 2 - BIG_POINT / 2;
  printf (FONT1, BIG_POINT);
  printf ("gsave (%s) col1 %d %d %d ctextu grestore\n", Userid, X_INDENT, y, DX);

/* print other banner page parameters */

/* sequence number */
#ifdef USE_WITH_PLP
  printf ("%s", FONT5);
  printf ("520 20 moveto\n");
  printf ("(%s)show\n", Requestid);
  printf ("520 800 moveto\n");
  printf ("(%s)show\n", Requestid);
  printf ("27 20 moveto\n");
  printf ("(%s)show\n", Requestid);
  printf ("27 800 moveto\n");
  printf ("(%s)show\n", Requestid);
#endif

  x = X_INDENT;
  y -= (MED_POINT * 2 + 220);
  printf ("%d %d moveto\n", x, y);
  send ("/Bookman-Demi findfont 75 scalefont setfont\n");
  printf ("(end of job) col4 %d %d %d ctextu\n", X_INDENT, y, DX);
  send ("showpage\n");
  send ("%%EOF\n");
  exit (JOB_IS_DONE);
}


/****************************************************************************
 *	send font selection information and text.			    *
 ****************************************************************************
 */

send (s)
    char *s;
{
    while (*s)
	fputc (*s++, outfp);
}

sendnormal (s)
    char *s;
{
    printf (FONT3, SMALL_POINT);
    send ("(");
    send (s);
    send (")show\n");
}

sendbold (s)
    char *s;
{
    printf (FONT4, SMALL_POINT);
    send ("(");
    send (s);
    send (")show\n");
}

/****************************************************************************
 *	trim ()								    *
 *	trucate garbage							    *
 ****************************************************************************
 */

void
trim (s)			/* trim trailing blanks  and \n */
    char *s;
{
    int many;
    
    for (many = strlen (s) - 1; many >= 0; many--)
	{
	    if (isgraph (s[many]))
		break;
	    else
		s[many] = '\0';
	}
}

/****************************************************************************
 *	bad_file()							    *
 ****************************************************************************
 */

bad_file ()
{
  fprintf (stderr, "Bad %s file\n", defref);
  exit (DISCARD_JOB);
}

/****************************************************************************
 *	compare ()							    *
 *	partial compare of two strings.					    *
 ****************************************************************************
*/

compare (a, b)
    char *a, *b;
{
    int aa, bb;

    while (*a && *b)
	{
	    aa = *a++;
	    bb = *b++;
	    if (UCCHAR (aa) != UCCHAR (bb))
		return (aa - bb);
	}
    return (0);
}


/****************************************************************************
 *	replacement_fonts()					            *
 *	read the text translation strings only.				    *
 ****************************************************************************
*/

replacement_fonts ()
{
    int found, xold, xnew;
    char line[201];
    char *x;
    
    if (pdef == NULL)
	return;
    found = 0;
    while (fgets (line, 200, pdef))	/* skip printer name     */
	{
	    if (strncmp (line, "%%PsfStart", 10) == 0)
		{
		    found = 1;
		    break;
		}
	}
    if (found)
	{
	    while (fgets (line, 200, pdef))	/* read fonts, translate */
		{
		    if (strncmp (line, "%%PsfEnd", 8) == 0)
			break;
		    if (strncmp (line, "%%translate", 11) == 0)
			{
			    x = strtok (line + 11, " ");
			    do
				{
				    xold = otoi (x) % 256;
				    xnew = otoi (x + 4) % 256;
				    xlate[xold] = xnew;
				}
			    while (x = strtok (NULL, " "));
			    need_xlate = 1;
			}
		}
	    fclose (pdef);
	    pdef = NULL;
	}
    else
	{
	    fclose (pdef);
	    pdef = NULL;
	}
    return;
}

/****************************************************************************
 *	octal to integer						    *
 ****************************************************************************
*/

int
otoi (s)
    char *s;
{
    int v, p, quit;
    
    v = 0;
    while (*s == ' ')
	s++;
    quit = 0;
    while (*s)
	{
	    switch (*s)
		{
		case '0':
		    p = 0;
		    break;
		case '1':
		    p = 1;
		    break;
		case '2':
		    p = 2;
		    break;
		case '3':
		    p = 3;
		    break;
		case '4':
		    p = 4;
		    break;
		case '5':
		    p = 5;
		    break;
		case '6':
		    p = 6;
		    break;
		case '7':
		    p = 7;
		    break;
		default:
		    quit = 1;
		}
	    if (quit)
		break;
	    v = (v << 3) + p;
	}
    return (v);
}


/****************************************************************************
 *	xlate_string ()							    *
 *	translate the string according to the language translation vector   *
 ****************************************************************************
*/
xlate_string (s)
    unsigned char *s;
{
    int sold;
    
    while (*s)
	{
	    sold = *s;
	    *s = xlate[sold];
	    s++;
    }
}


/****************************************************************************
 *	usage()								    *
 ****************************************************************************
 */
usage ()
{
    fprintf (stderr, "Usage:  psfcendbanner Userid Name Requestid Printer Options Date\n");
    fprintf (stderr, "                    Machineid Title [-b n] [-g size] [-u f.def]\n");
    fprintf (stderr, "        where:   Userid, Name, etc are passed by lpsched\n");
    fprintf (stderr, "                 -b n    = select paper bin n\n");
    fprintf (stderr, "                 -g size = select paper size (letter, b5, etc)\n");
    fprintf (stderr, "                -u f.def = use this printer definition file\n");
    exit (DISCARD_JOB);
}
