/***************************************************************************
 * U. Minnesota LPD Software * Copyright 1987, 1988, Patrick Powell
 * version 3.3.0 Justin Mason July 1994
 ***************************************************************************
 * MODULE: find_name.c
 * manages the Parms[] array entries.  Used by sender and receiver
 * file transfer functions of LPD.
 * The Parms[] array is maintained as a sorted list.
 * Find_name( char *name ): search the Parm[].filename entries for a match.
 *	returns: index if found, -1 if not;
 * Add_name( char *name ): search the Parm[].filename entries for a match.
 *	if not found, add the entry.
 *	returns: index where found or created.
 ***************************************************************************/

#include "lp.h"
#include "library/errormsg.h"

/***************************************************************************
 * Find_name( char *name)
 * The Parms[] array is used to hold the names of data and control files
 * corresponding to a job.  The Find_name will look through the list for
 * the job.
 ***************************************************************************/
int
Find_name (char *name) {
    int i, j;

    j = -1;
    for (i = 0; i < Parmcount; ++i) {
	if (strsame (Parms[i].filename, name)) {
	    j = i;
	    break;
	}
    }
	DEBUG6 ("Find_name: %s, entry %d", name, j);
    return (j);
}

/***************************************************************************
 * Add_name(char *name)
 * add a name to the Parms[i].filename array.
 ***************************************************************************/

int
Add_name (char *name) {
    int i;

    allocParmsIfNeeded ();
    if ((i = Find_name (name)) < 0) {
	i = Parmcount;
	(void) bzero ((char *) &Parms[i], sizeof (struct parm));
	(void) strcpy (Parms[i].filename, name);
	++Parmcount;
	growParmsIfNeeded ();
    }

	DEBUG5 ("Add_name: %s, entry %d", name, i);
    return (i);
}
