/***************************************************************************
 * U. Minnesota LPD Software * Copyright 1987, 1988, Patrick Powell
 * version 3.3.0 Justin Mason July 1994
 ***************************************************************************
 * MODULE: global.c
 * Definitions of global variables.
 * NOTE: system dependent,  default values defined in lp.h
 ***************************************************************************/

#include "lp.h"

/*************************************************************************
 * global variables
 *************************************************************************/

char *Name;			/* program Name (argv[0]) */
char *Printer;			/* Printer Name, set for a queue handler */

char *Newjobnum;                /* A number for a new command */
int mistake;                    /* var for switching */

char ShortHost[HOSTLEN];	/* Host machine name  */
char Host[HOSTLEN];		/* Host machine name (fully-qualified) */

char *Architecture = NULL;	/* Host machine architecture */

char *From = NULL;		/* client's machine name */
char *FQDN = NULL;		/* client's machine name (fully-qualified) */

char *Person;			/* Name of Person making request */
FILE *Lfd;			/* lock file stdio file descriptor */
char *First_name;		/* first name in printcap entry */
char *All_names;
int Debug;			/* Debugging level */
int Echo_on_stdout;		/* error messages on stdout and stderr */
int Errorcode = 2;		/* exit() value for fatal termination */

char Last_errormsg[BUFSIZ];	/* last errormessage */
struct plp_queue *Jobentry;	/* current job entry in queue */
int Request;			/* current lpd request code */
int Print_fd;			/* output printer file descriptor */
char CFparm[26][MAXPARMLEN + 1];/* parameters read from control file */
struct plp_queue *Queue;	/* Getq generates a vector of jobs */
struct plp_queue *Queue_tmp;    /* Needed by banner for seq.number */
                                /* without changing the act. value */
char *save_cfname;              /* needed by lpr for save copies */
char *save_dfname;              /* needed by lpr for save copies */
int Jobcount;			/* and returns the number of jobs  */
int Queue_format = REQ_DSHORT;	/* Queue status format */
int Is_root;			/* is this the root user? */
int Is_local;			/* doing this locally or remotely? */

struct stat LO_statb;		/* lockfile stat buffer */

uid_t DaemonUID;		/* daemon UID */
gid_t DaemonGID;		/* daemon GID */

struct parm *Parms = NULL;	/* character parameters */
int Parmcount;			/* number of requests */
unsigned int ParmSz = 0;	/* size of the Parm array (in struct parms) */
char buf_for_sort[BUFSIZ];      /* buffer for sorting (needed in utils.c) */
char unattached_printer[MAXPARMLEN]; /* Name of the unattached printer */
char attached_printer[MAXPARMLEN]; /* Name of the attached printer */

/******************************************************************
 * Definitions for the printcap variables.
 ******************************************************************/

int AB;				/* Always have a banner flag, ignore lpr -h option */
char *AF;			/* accounting file */
int AR;				/* remote accounting */
int BB;                         /* no banner pages should be printed between jobs */
int BK;				/* Berkeley compatible remote and local */
char *BP;			/* banner printer filter */
int BR;				/* baud rate if lp is a tty */
int CL;				/* close (& re-open) output device between jobs */
int CO;				/* cost of printing in dollars per thousand pages */
char *DS;			/* description of printer, may be needed by lpstat */
char *EP;			/* end printer filter */
int FB;				/* use BSD ifilter+ofilter semantics */
int FC;				/* flags to clear if lp is a tty */
int FD;				/* No forwarding, accept only from original site */
char *FF;			/* form feed string */
int FJ;				/* send control file  first to remote site */
int FO;				/* print a form feed when device is opened */
int FQ;				/* form feed on quitting */
int FS;				/* flags to set if lp is a tty */
char *FX;			/* allowable formats */
int HL;				/* banner at end-of-job? */
int LA;				/* local accounting */
char *LD;			/* leader string on opening */
char *LF;			/* log file for error messages */
char *LI;			/* file size limitation (value or filename) */
char *LN;			/* group allowed to use links */
char *LO;			/* lock file Name */
char *LP;			/* line Printer device Name */
int LS;				/* flag for banner page in the lprps package */
int MC;				/* maximum number of copies allowed */
char *MI;			/* minimum free space to leave on spool filesystem */
unsigned long MX;		/* maximum number of blocks to copy */
int NB;                         /* no endbanner page should be printed */
int NU;				/* not usable as a "real" printer queue */
int NW;				/* Networked file system */
char *OH;			/* only for the named host */
char *OP;			/* options for the pr program */
int PL;				/* page length */
char *PR;			/* pr program name */
char *PS;			/* Printer status file name */
char *PT;			/* printer type, may be neede by lpstat */
int PW;				/* page width */
int PX;				/* page width in pixels */
int PY;				/* page length in pixels */
char *QH;			/* queue handler */
int RB;                         /* sending banner page to the remote printer */
char *RM;			/* remote machine Name */
char *RP;			/* remote Printer Name */
int RT;				/* max retries */
int RW;				/* open LP for reading and writing */
int SA;                         /* indicates that this printer is a save area */
int SB;				/* short banner instead of normal header */
int SC;				/* suppress multiple copies */
char *SD;			/* spool directory */
int SF;				/* suppress FF on each print job */
int SH;				/* suppress header page */
char *SR; 			/* default format for this queue (f, l or p) */
char *SS;			/* name of queue that the server serves  */
char *ST;			/* status file Name */
char *SV;			/* names of servers */
int SZ;                         /* flag for sorting jobs according to their size */
char *TR;			/* trailer string to be output when Q empties */
char *TY;			/* terminal characteristics, stty options */
int XC;				/* flags to clear for local mode */
int XS;				/* flags to set for local mode */
char *XU;			/* restrict use to users with perms in file */
char *XT;			/* check format in string for printable text only */

/*
 * EUCS: Edinburgh University additions.
 */

char *CM;			/* EUCS: status line comment */
int DB;				/* EUCS: set debug level for queue handler */
int EU;				/* EUCS: Edinburgh Univ. extended control file format */
int ML;				/* min legal chars for printable */
char *MS;			/* EUCS: terminal characteristics, Sun lpd's "ty" */
int ED;				/* EUCS: expensive device (only jobs from trusted hosts) */
char *FL;			/* EUCS: list of valid forms */
int NC;				/* No charge queue - restrict access */

/* @@@ Fix from Dave Atkin */
pid_t qh_pid;                   /* PID of queue handler process */
/* end fix */

/*
 * filter arguments are indexed by first letter of the filter Name. For example:
 * "if=/usr/lib/iffilter" -> Filter_name['i'-'a']
 */
char *Filter_name[26];
/*
 * Prefilter arguments are indexed by first letter of the filter Name, as for the filters
 */
char *Prefilter_name[26];

int prmsline;		/* line of printer_perms file to be updated by acct filter */

/*************************************************************************
 * Printcap Entries,  Defaults, and Variables
 * The All_pc_vars[] and Status_pc_vars[] arrays contain 
 * - the Names of the printcap entries, 
 * - the type of entry,
 * - the default value and 
 * - the variable which is set by the entry.
 * The array is sorted by the printcap entry.
 *************************************************************************/

PC_ENTRY All_pc_vars[] = {
    {"ab", PC_FLAG, 0, 0, (char **) &AB, 0},
    {"ae", PC_STRING, 0, 0, &Prefilter_name['a' - 'a'], 0},
    {"af", PC_STRING, 0, 0, &AF, 0},
    {"ar", PC_FLAG, True, 0, (char **) &AR, 0},
    {"bb", PC_FLAG, 0, 0, (char **) &BB, 0},
    {"be", PC_STRING, 0, 0, &Prefilter_name['b' - 'a'], 0},
    {"bf", PC_STRING, 0, 0, &Filter_name['b' - 'a'], 0},
    {"bk", PC_FLAG, 0, 0, (char **) &BK, 0},
    {"bp", PC_STRING, 0, 0, &BP, 0},
    {"br", PC_NUM, 0, 0, (char **) &BR, 0},
    {"ce", PC_STRING, 0, 0, &Prefilter_name['c' - 'a'], 0},
    {"cf", PC_STRING, 0, 0, &Filter_name['c' - 'a'], 0},
    {"cl", PC_FLAG, 0, 0, (char **) &CL, 0},
    {"cm", PC_STRING, 0, 0, &CM, 0},
    {"co", PC_NUM, 0, 0, (char **) &CO, 0},
    {"db", PC_NUM, 0, 0, (char **) &DB, 0},
    {"de", PC_STRING, 0, 0, &Prefilter_name['d' - 'a'], 0},
    {"df", PC_STRING, 0, 0, &Filter_name['d' - 'a'], 0},
    {"ds", PC_STRING, 0, 0, &DS, 0},
    {"ed", PC_FLAG, 0, 0, (char **) &ED, 0},
    {"eu", PC_FLAG, 0, 0, (char **) &EU, 0},
    {"ee", PC_STRING, 0, 0, &Prefilter_name['e' - 'a'], 0},
    {"ef", PC_STRING, 0, 0, &Filter_name['e' - 'a'], 0},
    {"ep", PC_STRING, 0, 0, &EP, 0},
    {"eu", PC_FLAG, 0, 0, (char **) &EU, 0},
    {"fb", PC_FLAG, True, 0, (char **) &FB, 0},
    {"fc", PC_NUM, 0, 0, (char **) &FC, 0},
    {"fd", PC_FLAG, 0, 0, (char **) &FD, 0},
    {"fe", PC_STRING, 0, 0, &Prefilter_name['f' - 'a'], 0},
    {"ff", PC_STRING, 0, DEFFF, &FF, 0},
    {"fj", PC_FLAG, 0, 0, (char **) &FJ, 0},
    {"fl", PC_STRING, 0, 0, &FL, 0},
    {"fo", PC_FLAG, 0, 0, (char **) &FO, 0},
    {"fq", PC_FLAG, 0, 0, (char **) &FQ, 0},
    {"fs", PC_NUM, 0, 0, (char **) &FS, 0},
    {"fx", PC_STRING, 0, 0, &FX, 0},
    {"ge", PC_STRING, 0, 0, &Prefilter_name['g' - 'a'], 0},
    {"gf", PC_STRING, 0, 0, &Filter_name['g' - 'a'], 0},
    {"he", PC_STRING, 0, 0, &Prefilter_name['h' - 'a'], 0},
    {"hf", PC_STRING, 0, 0, &Filter_name['h' - 'a'], 0},
    {"hl", PC_FLAG, 0, 0, (char **) &HL, 0},
    {"ie", PC_STRING, 0, 0, &Prefilter_name['i' - 'a'], 0},
    {"if", PC_STRING, 0, 0, &Filter_name['i' - 'a'], 0},
    {"je", PC_STRING, 0, 0, &Prefilter_name['j' - 'a'], 0},
    {"jf", PC_STRING, 0, 0, &Filter_name['j' - 'a'], 0},
    {"ke", PC_STRING, 0, 0, &Prefilter_name['k' - 'a'], 0},
    {"kf", PC_STRING, 0, 0, &Filter_name['k' - 'a'], 0},
    {"la", PC_FLAG, True, 0, (char **) &LA, 0},
    {"ld", PC_STRING, 0, 0, &LD, 0},
    {"le", PC_STRING, 0, 0, &Prefilter_name['l' - 'a'], 0},
    {"lf", PC_STRING, 0, DEFLOGF, &LF, 0},
    {"li", PC_STRING, 0, 0, &LI, 0},
    {"ln", PC_STRING, 0, 0, &LN, 0},
    {"lo", PC_STRING, 0, DEFLOCK, &LO, 0},
    {"lp", PC_STRING, 0, 0, &LP, 0},
    {"ls", PC_FLAG, 0, 0, (char **) &LS, 0},
    {"mc", PC_NUM, DEFMAXCOPIES, 0, (char **) &MC, 0},
    {"me", PC_STRING, 0, 0, &Prefilter_name['m' - 'a'], 0},
    {"mf", PC_STRING, 0, 0, &Filter_name['m' - 'a'], 0},
    {"mi", PC_NUM, 0, 0, (char **) &MI, 0},
    {"ml", PC_NUM, DEFMINLEGAL, 0, (char **) &ML, 0},
    {"ms", PC_STRING, 0, 0, (char **) &MS, 0},
    {"mx", PC_LONG, DEFMX, 0, (char **) &MX, 0},
    {"nb", PC_FLAG, 0, 0, (char **) &NB, 0},
    {"nc", PC_FLAG, 0, 0, (char **) &NC, 0},
    {"ne", PC_STRING, 0, 0, &Prefilter_name['n' - 'a'], 0},
    {"nf", PC_STRING, 0, 0, &Filter_name['n' - 'a'], 0},
    {"nu", PC_FLAG, False, 0, (char **) &NU, 0},
    {"nw", PC_FLAG, 0, 0, (char **) &NW, 0},
    {"oe", PC_STRING, 0, 0, &Prefilter_name['o' - 'a'], 0},
    {"of", PC_STRING, 0, 0, &Filter_name['o' - 'a'], 0},
    {"oh", PC_STRING, 0, 0, &OH, 0},
    {"op", PC_STRING, 0, 0, &OP, 0},
    {"pe", PC_STRING, 0, 0, &Prefilter_name['p' - 'a'], 0},
    {"pf", PC_STRING, 0, 0, &Filter_name['p' - 'a'], 0},
    {"pl", PC_NUM, DEFLENGTH, 0, (char **) &PL, 0},
    {"pr", PC_STRING, 0, DEFPR, &PR, 0},
    {"ps", PC_STRING, 0, DEFPRSTAT, &PS, 0},
    {"pt", PC_STRING, 0, 0, &PT, 0},
    {"pw", PC_NUM, DEFWIDTH, 0, (char **) &PW, 0},
    {"px", PC_NUM, 0, 0, (char **) &PX, 0},
    {"py", PC_NUM, 0, 0, (char **) &PY, 0},
    {"qe", PC_STRING, 0, 0, &Prefilter_name['q' - 'a'], 0},
    {"qf", PC_STRING, 0, 0, &Filter_name['q' - 'a'], 0},
    {"qh", PC_STRING, 0, 0, &QH, 0},
    {"rb", PC_FLAG, 0, 0, (char **) &RB, 0},
    {"re", PC_STRING, 0, 0, &Prefilter_name['r' - 'a'], 0},
    {"rf", PC_STRING, 0, 0, &Filter_name['r' - 'a'], 0},
    {"rm", PC_STRING, 0, 0, &RM, 0},
    {"rp", PC_STRING, 0, 0, &RP, 0},
    {"rt", PC_NUM, DEFRETRY, 0, (char **) &RT, 0},
    {"rw", PC_FLAG, 0, 0, (char **) &RW, 0},
    {"sa", PC_FLAG, 0, 0, (char **) &SA, 0},
    {"sb", PC_FLAG, 0, 0, (char **) &SB, 0},
    {"sc", PC_FLAG, 0, 0, (char **) &SC, 0},
    {"sd", PC_STRING, 0, 0, &SD, 0},
    {"se", PC_STRING, 0, 0, &Prefilter_name['s' - 'a'], 0},
    {"sf", PC_FLAG, 0, 0, (char **) &SF, 0},
    {"sh", PC_FLAG, 0, 0, (char **) &SH, 0},
    {"sr", PC_STRING, 0, 0, &SR, 0},
    {"ss", PC_STRING, 0, 0, &SS, 0},
    {"st", PC_STRING, 0, DEFSTAT, &ST, 0},
    {"sv", PC_STRING, 0, 0, &SV, 0},
    {"sz", PC_FLAG, 0, 0, (char **) &SZ, 0},
    {"te", PC_STRING, 0, 0, &Prefilter_name['t' - 'a'], 0},
    {"tf", PC_STRING, 0, 0, &Filter_name['t' - 'a'], 0},
    {"tr", PC_STRING, 0, 0, &TR, 0},
    {"ty", PC_STRING, 0, 0, &TY, 0},
    {"ue", PC_STRING, 0, 0, &Prefilter_name['u' - 'a'], 0},
    {"uf", PC_STRING, 0, 0, &Filter_name['u' - 'a'], 0},
    {"ve", PC_STRING, 0, 0, &Prefilter_name['v' - 'a'], 0},
    {"vf", PC_STRING, 0, 0, &Filter_name['v' - 'a'], 0},
    {"we", PC_STRING, 0, 0, &Prefilter_name['w' - 'a'], 0},
    {"wf", PC_STRING, 0, 0, &Filter_name['w' - 'a'], 0},
    {"xc", PC_NUM, 0, 0, (char **) &XC, 0},
    {"xe", PC_STRING, 0, 0, &Prefilter_name['x' - 'a'], 0},
    {"xf", PC_STRING, 0, 0, &Filter_name['x' - 'a'], 0},
    {"xs", PC_NUM, 0, 0, (char **) &XS, 0},
    {"xt", PC_STRING, 0, 0, &XT, 0},
    {"xu", PC_STRING, 0, 0, &XU, 0},
    {"ye", PC_STRING, 0, 0, &Prefilter_name['y' - 'a'], 0},
    {"yf", PC_STRING, 0, 0, &Filter_name['y' - 'a'], 0},
    {"ze", PC_STRING, 0, 0, &Prefilter_name['z' - 'a'], 0},
    {"zf", PC_STRING, 0, 0, &Filter_name['z' - 'a'], 0}
};

int All_pc_len = sizeof (All_pc_vars) / sizeof (PC_ENTRY);

/**************************************************************************
 * Status_pc_vars[]
 * status checking uses only a subset of all the printcap entries
 **************************************************************************/

PC_ENTRY Status_pc_vars[] = {
    {"af", PC_STRING, 0, 0, &AF, 0},
    {"cl", PC_FLAG, 0, 0, (char **) &CL, 0},
    {"cm", PC_STRING, 0, 0, &CM, 0},
    {"co", PC_NUM, 0, 0, (char **) &CO, 0},
    {"db", PC_NUM, 0, 0, (char **) &DB, 0},
    {"ds", PC_STRING, 0, 0, &DS, 0},
    {"ed", PC_FLAG, 0, 0, (char **) &ED, 0},
    {"eu", PC_FLAG, 0, 0, (char **) &EU, 0},
    {"fd", PC_FLAG, 0, 0, (char **) &FD, 0},
    {"fl", PC_STRING, 0, 0, &FL, 0},
    {"lf", PC_STRING, 0, DEFLOGF, &LF, 0},
    {"lo", PC_STRING, 0, DEFLOCK, &LO, 0},
    {"lp", PC_STRING, 0, 0, &LP, 0},
    {"mx", PC_NUM, DEFMX, 0, (char **) &MX, 0},
    {"nc", PC_FLAG, 0, 0, (char **) &NC, 0},
    {"nu", PC_FLAG, False, 0, (char **) &NU, 0},
    {"nw", PC_FLAG, 0, 0, (char **) &NW, 0},
    {"oh", PC_STRING, 0, 0, &OH, 0},
    {"ps", PC_STRING, 0, DEFPRSTAT, &PS, 0},
    {"pt", PC_STRING, 0, 0, &PT, 0},
    {"rm", PC_STRING, 0, 0, &RM, 0},
    {"rp", PC_STRING, 0, 0, &RP, 0},
    {"sa", PC_FLAG, 0, 0, (char **) &SA, 0},
    {"sd", PC_STRING, 0, 0, &SD, 0},
    {"ss", PC_STRING, 0, 0, &SS, 0},
    {"st", PC_STRING, 0, DEFSTAT, &ST, 0},
    {"sv", PC_STRING, 0, 0, &SV, 0},
    {"sz", PC_FLAG, 0, 0, (char **) &SZ, 0},
    {"xu", PC_STRING, 0, 0, &XU, 0}
};

int Status_pc_len = sizeof (Status_pc_vars) / sizeof (PC_ENTRY);

/**************************************************************************
 * Server_pc_vars[]
 * just used to get a select set of server information
 **************************************************************************/
PC_ENTRY Server_pc_vars[] = {
    {"db", PC_NUM, 0, 0, (char **) &DB, 0},
    {"cm", PC_STRING, 0, 0, &CM, 0},
    {"nu", PC_FLAG, False, 0, (char **) &NU, 0},
    {"oh", PC_STRING, 0, 0, &OH, 0},
    {"ps", PC_STRING, 0, DEFPRSTAT, &PS, 0},
    {"sa", PC_FLAG, 0, 0, (char **) &SA, 0},
    {"ss", PC_STRING, 0, 0, &SS, 0},
    {"st", PC_STRING, 0, DEFSTAT, &ST, 0},
    {"sv", PC_STRING, 0, 0, &SV, 0}
};

int Server_pc_len = sizeof (Server_pc_vars) / sizeof (PC_ENTRY);

char *Lpdlogf = NULL;
char *Masterlock = NULL;
int Lpr_port_num;		/* port number to connect to */

int Maxportno, Minportno;

#ifdef HAVE_SYSLOG_H
#include <syslog.h>
#else
#define	LOG_ERR		4	/* synonym of LOG_ERROR */
#define	LOG_CRIT	5	/* critical information */
#define	LOG_WARNING	6	/* warning messages */
#define	LOG_NOTICE	7	/* important information */
#define	LOG_INFO	8	/* informational message */
#define	LOG_DEBUG	9	/* Debug level info */
#endif

int XLOG_ERR = LOG_ERR;		/* synonym of LOG_ERROR */
int XLOG_CRIT = LOG_CRIT;	/* critical information */
int XLOG_WARNING = LOG_WARNING;	/* warning messages */
int XLOG_NOTICE = LOG_NOTICE;	/* important information */
int XLOG_INFO = LOG_INFO;	/* informational message */
int XLOG_DEBUG = LOG_DEBUG;	/* Debug level info */

/* Printcap configuration stuff */
int num_confs = 0;		/* number of entries in pcap.conf file */
struct conf_entry conf[MAX_CONFS];	/* the entries in pcap.conf */

/*
 * Configuration file
 * 
 */

#ifdef LOCAL
char *Permfile_path = NULL;
char *Printcap_path = NULL;
#endif

/* more Edinburgh stuff: */

char *Zoptions_path = NULL;
char *Costcode_path = NULL;
char *Forms_active = NULL;
char *Forms_file = NULL;

#ifdef HESIOD
int *Hesiod_cross_domain = NULL;
char *Hesiod_printcap_key = NULL;
char *Hesiod_costcode_key = NULL;
#endif

#ifdef NIS
char *NIS_printcap_bykey = NULL;
char *NIS_printcap_byname = NULL;
char *NIS_costcode_bykey = NULL;
char *NIS_costcode_byname = NULL;
#endif

char *Errors_file = NULL, *Attach_file = NULL, *ConfigFile = NULL,
    *Daemon_user = NULL, *Mail_command = NULL,
    *Mail_addressing = NULL, *Mail_from_address = NULL,
    *DefaultPrintcap = NULL, *Ctrlfile_logdir = NULL,
    *Filter_path = NULL, *Filter_LD_path = NULL,
    *Printcap_include_path = NULL, *Config_include_path = NULL,
    *Permfile_include_path = NULL, *Printer_port = NULL,
    *Host_domain = NULL, *AnonUsername = NULL, *SaveCopyArea = NULL;

char AllowNonPrivPorts, LogNonPrivPorts, Mail_on_errors, Restrict_symlinks,
    Log_no_filenames, Log_LPRMs, Log_LPQs, Log_LPCs, Log_LPRs,
    Check_for_nonprintable, Show_empty_queues, EUCS_banner_format,
    DomainNameUnset, Use_printcap_cache, Use_perm_cache, jobs_sorted_on_size,
    DbgPcap, DbgRem, DbgLocal, DbgConf, DbgPerms, Use_first_printer, Use_history;

char *SafeEnviron[MAXUSERENVIRON];	/* the current environment */

#ifdef SETPROCTITLE
char **Argv = NULL;			/* pointer to argument vector */
char *LastArgv = NULL;			/* end of argv */
#endif

char *Malloc_failed_msg = "malloc failed -- out of memory";

char *Remote_user_auth;

int ThisIsLPD;				/* for library/common functions */
int called_by_lpr = 0;
char rank_for_lpr[10];
int Job_number_by_lpr;
short tk_interface_used = 0;
short called_from_lprm = 0;
short called_from_one_job = 0;
short lprm_called = 0;
short mail_to_user_for_remote_remove = 0;
short job_moved = 0;
int interval = 1;
short sysv_lp = 0;                      /* Flag if the programs were called in
					 * "sys v mode"
					 */
short forms_info_printed = 0;           /* Flag to avoid having the forms info
					 * displayed more then once
					 */
char Use_with_cde;
int no_bb_banner;
unsigned long spooled_time;             /* Time when job was spooled */
unsigned long printed_time;             /* Time when job was printed */

char jobname_bb[128];            /* for use with banner programs and */
char bnrname_bb[128];            /* the BB flags used in setup_filter.c */
char username_bb[128];
char seqnumber_bb[5];
char hostname_bb[128];
