/***************************************************************************
 * U. Minnesota LPD Software * Copyright 1987, 1988, Patrick Powell
 * version 3.3.0 Justin Mason July 1994
 * version 4.1.2 Stefano Ianigro June 1998
 ***************************************************************************
 * MODULE: Remote_ops.c
 * intiate remote operations from the local machine
 ***************************************************************************/

#include "lp.h"
#include "library/errormsg.h"

/***************************************************************************
 * Remote_status()
 * get the status from the remote Host RM for Printer RP
 ***************************************************************************/
void
Remote_status (void) {
    char buf[BUFSIZ];

    if (RP == 0 || *RP == 0) {
	fatal (XLOG_INFO, "no RP specified for RM (%s)", RM);
    }
    (void) sprintf (buf, "%c%s\n", Queue_format, RP);
    DEBUG3 ("Remote_status: %s, '%d' %s", RM, buf[0], buf + 1);
    if (JSUCC == Link_line (3, buf)) {
	    DBGREM4 ("request sent");
	Link_get ();
    } else {
	(void) fprintf (stdout, "Remote connect to '%s' (%s) failed: %s\n",
			RP, RM, Errormsg (errno));
    }
    Link_close ();
    return;
}
/***************************************************************************
 * Remote_history (char *)
 * get the history from the remote Host RM for Printer RP
 * Note:
 *  This function gets the history information via the history operation
 *  (code 9 )from the remote printer; it only works together with a PLP
 *  daemon of this package. Otherwise you get an "illegal operation" back!
 * ---st, 26.08.1997 
 ***************************************************************************/
void
Remote_history (char *history_name) {
    char buf[BUFSIZ];

    if (RP == 0 || *RP == 0) {
	fatal (XLOG_INFO, "no RP specified for RM (%s)", RM);
    }
    /* if no username is given for history_name, we take that one in Person */
    if (strlen (history_name) == 0) {
	(void) strcpy (history_name, Person);
    }
    (void) sprintf (buf, "%c %s history %s %s\n", 9, Person, history_name, RP);
    DEBUG3 ("Remote_history: %s, '%d'%s", RM, buf[0], buf + 1);
    if (JSUCC == Link_line (9, buf)) {
	    DBGREM4 ("request sent");
	Link_get ();
    } else {
	(void) fprintf (stdout, "Remote connect to '%s' (%s) failed: %s\n",
			RP, RM, Errormsg (errno));
    }
    Link_close ();
    return;
}

/***************************************************************************
 * Remote_listforms (void)
 * get the forms loaded from the remote Host RM for Printer RP
 * Note:
 *  This function gets the listforms information via a control_operation
 *  (code 6) from the remote printer; it only works together with a PLP
 *  daemon of this package. Otherwise you get an "illegal operation" back!
 * ---st, 12.05.1998
 ***************************************************************************/
void
Remote_listforms (void) {
    char buf[BUFSIZ];

    if (RP == 0 || *RP == 0) {
	fatal (XLOG_INFO, "no RP specified for RM (%s)", RM);
    }
    (void) sprintf (buf, "%c %s listforms %s\n", 6, Person, RP);
    DEBUG3 ("Remote_listforms: %s, '%d'%s", RM, buf[0], buf + 1);
    if (JSUCC == Link_line (6, buf)) {
	    DBGREM4 ("request sent");
	Link_get ();
    } else {
	(void) fprintf (stdout, "Remote connect to '%s' (%s) failed: %s\n",
			RP, RM, Errormsg (errno));
    }
    Link_close ();
    return;
}

/***************************************************************************
 * Remote_remove()
 * send a remote remove request to the remote Host
 ***************************************************************************/
void
Remote_remove (void) {
    char buf[BUFSIZ];		/* holds command */
    char *bp, *ep;		/* ACME Pointers, Inc. */
    int i;			/* ACME Integers, Inc. */
    static char *xsp = " ";	/* needs a space */

    /*
     * set up the remote command format: <REQ_REMOVE>Printer Person [Parms]
     */
    bp = buf;
    ep = buf + sizeof (buf);
    *bp++ = REQ_REMOVE;
    if (RP == 0 || *RP == 0) {
	fatal (XLOG_INFO, "no RP specified for RM %s", RM);
    }
    bp = estrcp (bp, RP, ep);
    bp = estrcp (bp, xsp, ep);
    bp = estrcp (bp, Person, ep);
    for (i = 0; i < Parmcount; ++i) {
	bp = estrcp (bp, xsp, ep);
	bp = estrcp (bp, Parms[i].str, ep);
    }
    bp = estrcp (bp, "\n", ep);
    /*
     * check command line for length
     */
    if (bp == 0) {
	fatal (XLOG_INFO, "Remote_remove: command line too long '%s'");
    }
    /*
     * send command
     */
	DBGREM3 ("Remote_remove: %s, '%d'%s", RM, buf[0], buf + 1);
    if (JSUCC == Link_line (3, buf)) {
	    DBGREM3 ("request sent");
	Link_get ();
    }
    Link_close ();
    return;
}

/***************************************************************************
 * Remote_control(cmd, param)
 * send the remote command to the printer
 * This has the form: \006person command RP param
 ***************************************************************************/
void
Remote_control (char *cmd, char *param) {
    char buf[BUFSIZ];

    if (RP == 0 || *RP == 0) {
	fatal (XLOG_INFO, "no RP specified for RM (%s)", RM);
    }
    /* special format for history command */
    if (strsame (cmd, "history")) {
	(void) sprintf (buf, "%c%s %s %s %s\n", 
			REQ_CONTROL, Person, cmd, param, RP);
    } else {
    (void) sprintf (buf, "%c%s %s %s%s\n", 
		    REQ_CONTROL, Person, cmd, RP, param);
    }
	DBGREM3 ("Remote_control:%s '%d%'%s'", RM, buf[0], buf + 1);
    if (JSUCC == Link_line (3, buf)) {
	    DBGREM4 ("request sent");
	Link_get ();
    } else {
	(void) fprintf (stdout, "Remote connect to '%s' (%s) failed: %s\n",
			RP, RM, Errormsg (errno));
    }
    Link_close ();
    return;
}

/***************************************************************************
 * Remote_start()
 * start the remote printer
 ***************************************************************************/
int
Remote_start (void) {
    char buf[BUFSIZ];
    int succ;

    if (RP == 0 || *RP == 0) {
	fatal (XLOG_INFO, "no RP specified for RM (%s)", RM);
    }
    (void) sprintf (buf, "%c%s\n", REQ_START, RP);
	DBGREM3 ("Remote_start:%s '%d'\"%s\"", RM, buf[0], buf + 1);
    succ = Link_line (3, buf);
    if (JSUCC == succ) {
	    DBGREM4 ("request sent");
	(void) Link_confirm ();
    }
    Link_close ();
    return (JSUCC == succ);
}
